
-- ͻ
--        D E S I G N   E N G I N E R I N G              DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         Package     Os2.dlg                                        
--                                                                    
--         Author :  Leonid Dulman     1997                           
--                                                                    
--              GNU ADA RUNTIME LIBRARY (GNARL) COMPONENTS            
--                                                                    
--              OS/2 Presentation Manager CUA controls                
--                       and dialogs declarations                     
-- ͼ

with Os2.pmwb ;            use Os2.pmwb  ;
with Os2.pmwd ;            use Os2.pmwd  ;
with Os2.gpi  ;            use Os2.gpi   ;
with Os2.fm   ;            use Os2.fm  ;
with Interfaces.C;         use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;

package Os2.dlg is
-- pragma Preelaborate (dlg);
   --********************************************************************--
   --                                                                    --
   --                     F I L E    D I A L O G                         --
   --                                                                    --
   --********************************************************************--

   ------------------------------------------------------------------------
   -- File Dialog Invocation Flag Definitions.                           --
   ------------------------------------------------------------------------
 FDS_CENTER             :constant ULong := 16#00000001#; -- Center within owner wnd
 FDS_CUSTOM             :constant ULong := 16#00000002#; -- Use custom user template
 FDS_FILTERUNION        :constant ULong := 16#00000004#; -- Use union of filters
 FDS_HELPBUTTON         :constant ULong := 16#00000008#; -- Display Help button
 FDS_APPLYBUTTON        :constant ULong := 16#00000010#; -- Display Apply button
 FDS_PRELOAD_VOLINFO    :constant ULong := 16#00000020#; -- Preload volume info
 FDS_MODELESS           :constant ULong := 16#00000040#; -- Make dialog modeless
 FDS_INCLUDE_EAS        :constant ULong := 16#00000080#; -- Always load EA info
 FDS_OPEN_DIALOG        :constant ULong := 16#00000100#; -- Select Open dialog
 FDS_SAVEAS_DIALOG      :constant ULong := 16#00000200#; -- Select SaveAs dialog
 FDS_MULTIPLESEL        :constant ULong := 16#00000400#; -- Enable multiple selection
 FDS_ENABLEFILELB       :constant ULong := 16#00000800#; -- Enable SaveAs Listbox

   ------------------------------------------------------------------------
   -- File Dialog Selection returned attribute                           --
   ------------------------------------------------------------------------
FDS_EFSELECTION      :constant Long :=   0 ;
FDS_LBSELECTION      :constant Long :=   1  ;

   ------------------------------------------------------------------------
   -- Error Return Codes from dialog (self defining)                     --
   ------------------------------------------------------------------------
FDS_SUCCESSFUL                 :constant Long :=   0 ;
FDS_ERR_DEALLOCATE_MEMORY      :constant Long :=   1 ;
FDS_ERR_FILTER_TRUNC           :constant Long :=   2 ;
FDS_ERR_INVALID_DIALOG         :constant Long :=   3 ;
FDS_ERR_INVALID_DRIVE          :constant Long :=   4 ;
FDS_ERR_INVALID_FILTER         :constant Long :=   5 ;
FDS_ERR_INVALID_PATHFILE       :constant Long :=   6 ;
FDS_ERR_OUT_OF_MEMORY          :constant Long :=   7 ;
FDS_ERR_PATH_TOO_LONG          :constant Long :=   8 ;
FDS_ERR_TOO_MANY_FILE_TYPES    :constant Long :=   9 ;
FDS_ERR_INVALID_VERSION        :constant Long :=   10;
FDS_ERR_INVALID_CUSTOM_HANDLE  :constant Long :=   11;
FDS_ERR_DIALOG_LOAD_ERROR      :constant Long :=   12;
FDS_ERR_DRIVE_ERROR            :constant Long :=   13;

   ------------------------------------------------------------------------
   -- File Dialog Messages.                                              --
   ------------------------------------------------------------------------
FDM_FILTER  :constant ULong :=  WM_USER+40; -- mp1 := PSZ pszFileName         --
                                           -- mp2 := PSZ EA .TYPE value      --
                                           -- mr  := 1 . keep file.      --
FDM_VALIDATE:constant ULong :=  WM_USER+41; -- mp1 := PSZ pszPathName         --
                                           -- mp2 := USHORT Field name id    --
                                           -- mr  := 1 . Valid name      --
FDM_ERROR   :constant ULong :=  WM_USER+42; -- mp1 := USHORT Error message id --
                                           -- mp2 := NULL   reserved         --
                                           -- mr  := NULL . Use when others msg --

   ------------------------------------------------------------------------
   -- Define the type that is a pointer to an array of pointers.         --
   --     Hence =>  pointer to an array of Z string pointers.               --
   ------------------------------------------------------------------------
   type  APSZ is array(integer range <>) of PSZ ;
subtype  PAPSZ is PVOID;
-- subtype  FullFileStr is string(1..CCHMAXPATH); -- FullFilePath string

   ------------------------------------------------------------------------
   -- File Dialog application data structure.                            --
   ------------------------------------------------------------------------
   type FILEDLG is record    -- fildlg --
   cbSize          :ULONG ;  -- Size of FILEDLG structure.
   fl              :ULONG ;  -- FDS_ flags. Alter behavior of dlg.
   ulUser          :ULONG ;  -- User defined field.
   lReturn         :LONG  ;  -- Result code from dialog dismissal.
   lSRC            :LONG  ;  -- System return code.
   pszTitle        :PSZ   ;  -- String to display in title bar.
   pszOKButton     :PSZ   ;  -- String to display in OK button.
   pfnDlgProc      :PFNWP ;  -- Entry point to custom dialog proc.
   pszIType        :PSZ   ;  -- Pointer to string containing
      --                     -- initial EA type filter. Type
      --                     -- does not have to exist in list.
   papszITypeList  :PAPSZ;   -- Pointer to table of pointers that
                             -- point to null terminated Type
                             -- strings. End of table is marked
                             -- by a NULL pointer.
   pszIDrive       :PSZ  ;   -- Pointer to string containing
                             -- initial drive. Drive does not
                             -- have to exist in drive list.
   papszIDriveList :PAPSZ;   -- Pointer to table of pointers that
                             -- point to null terminated Drive
                             -- strings. End of table is marked
                             --  by a NULL pointer.
   hMod             :HMODULE;-- Custom File Dialog template.
   szFullFile       :FullFileStr;  -- Initial or selected fully
                                   -- qualified path and file.
   papszFQFilename  :PAPSZ ; -- Pointer to table of pointers that
                             -- point to null terminated FQFname
                             -- strings. End of table is marked
                             --  by a NULL pointer.
   ulFQFCount       :ULONG ; -- Number of files selected
   usDlgId          :USHORT; -- Custom dialog id.
   x                :SHORT ; -- X coordinate of the dialog
   y                :SHORT ; -- Y coordinate of the dialog
   sEAType          :SHORT ; -- Selected file's EA Type.
   end record;
   type PFILEDLG  is access all FILEDLG;

   ------------------------------------------------------------------------
   -- File Dialog - Function Prototype                                   --
   ------------------------------------------------------------------------
type TypeDlg is (OpenDialog,SaveAsDialog);
pszOpen     : constant string := "Open File" & Character'val(0)  ; -- Title of Open dialog
pszSaveAs   : constant string := "Save File" & Character'val(0)  ; -- Title of Save dialog

procedure InitDlgField(fild: out FILEDLG;            -- Dialog Recorrd
                       fo  : TypeDlg := OpenDialog); -- File Operation

function WinFileDlg(hwndP:HWND     ;
                     hwndO:HWND     ;
                     pfild:PFILEDLG ) return  HWND   ;
pragma Import(c,WinFileDlg, Link_name=>"_WinFileDlg");

function WinDefFileDlgProc(h_wnd:HWND   ;
                            msg :ULONG  ;
                            mp1 :MPARAM ;
                            mp2 :MPARAM ) return  MRESULT;
pragma Import(c,WinDefFileDlgProc, Link_name=>"_WinDefFileDlgProc");

function WinFreeFileDlgList(papszFQFilename:PAPSZ ) return  BOOL    ;
pragma Import(c,WinFreeFileDlgList, Link_name=>"_WinFreeFileDlgList");

   ------------------------------------------------------------------------
   -- File Dialog - dialog and control ids                               --
   ------------------------------------------------------------------------
DID_FILE_DIALOG    :constant ULong :=     256 ;
DID_FILENAME_TXT   :constant ULong :=     257 ;
DID_FILENAME_ED    :constant ULong :=     258 ;
DID_DRIVE_TXT      :constant ULong :=     259 ;
DID_DRIVE_CB       :constant ULong :=     260 ;
DID_FILTER_TXT     :constant ULong :=     261 ;
DID_FILTER_CB      :constant ULong :=     262 ;
DID_DIRECTORY_TXT  :constant ULong :=     263 ;
DID_DIRECTORY_LB   :constant ULong :=     264 ;
DID_FILES_TXT      :constant ULong :=     265 ;
DID_FILES_LB       :constant ULong :=     266 ;
DID_HELP_PB        :constant ULong :=     267 ;
DID_APPLY_PB       :constant ULong :=     268 ;
DID_OK_PB          :constant ULong :=     DID_OK ;
DID_CANCEL_PB      :constant ULong :=     DID_CANCEL ;

IDS_FILE_ALL_FILES_SELECTOR  :constant Long :=    1000 ;
IDS_FILE_BACK_CUR_PATH       :constant Long :=    1001 ;
IDS_FILE_BACK_PREV_PATH      :constant Long :=    1002 ;
IDS_FILE_BACK_SLASH          :constant Long :=    1003 ;
IDS_FILE_BASE_FILTER         :constant Long :=    1004 ;
IDS_FILE_BLANK               :constant Long :=    1005 ;
IDS_FILE_COLON               :constant Long :=    1006 ;
IDS_FILE_DOT                 :constant Long :=    1007 ;
IDS_FILE_DRIVE_LETTERS       :constant Long :=    1008 ;
IDS_FILE_FWD_CUR_PATH        :constant Long :=    1009 ;
IDS_FILE_FWD_PREV_PATH       :constant Long :=    1010 ;
IDS_FILE_FORWARD_SLASH       :constant Long :=    1011 ;
IDS_FILE_PARENT_DIR          :constant Long :=    1012 ;
IDS_FILE_Q_MARK              :constant Long :=    1013 ;
IDS_FILE_SPLAT               :constant Long :=    1014 ;
IDS_FILE_SPLAT_DOT           :constant Long :=    1015 ;
IDS_FILE_SAVEAS_TITLE        :constant Long :=    1016 ;
IDS_FILE_SAVEAS_FILTER_TXT   :constant Long :=    1017 ;
IDS_FILE_SAVEAS_FILENM_TXT   :constant Long :=    1018 ;
IDS_FILE_DUMMY_FILE_NAME     :constant Long :=    1019 ;
IDS_FILE_DUMMY_FILE_EXT      :constant Long :=    1020 ;
IDS_FILE_DUMMY_DRIVE         :constant Long :=    1021 ;
IDS_FILE_DUMMY_ROOT_DIR      :constant Long :=    1022 ;
IDS_FILE_PATH_PTR            :constant Long :=    1023 ;
IDS_FILE_VOLUME_PREFIX       :constant Long :=    1024 ;
IDS_FILE_VOLUME_SUFFIX       :constant Long :=    1025 ;
IDS_FILE_PATH_PTR2           :constant Long :=    1026 ;
IDS_FILE_INVALID_CHARS       :constant Long :=    1027 ;

IDS_FILE_BAD_DRIVE_NAME            :constant Long :=    1100 ;
IDS_FILE_BAD_DRIVE_OR_PATH_NAME    :constant Long :=    1101 ;
IDS_FILE_BAD_FILE_NAME             :constant Long :=    1102 ;
IDS_FILE_BAD_FQF                   :constant Long :=    1103 ;
IDS_FILE_BAD_NETWORK_NAME          :constant Long :=    1104 ;
IDS_FILE_BAD_SUB_DIR_NAME          :constant Long :=    1105 ;
IDS_FILE_DRIVE_NOT_AVAILABLE       :constant Long :=    1106 ;
IDS_FILE_FQFNAME_TOO_LONG          :constant Long :=    1107 ;
IDS_FILE_OPEN_DIALOG_NOTE          :constant Long :=    1108 ;
IDS_FILE_PATH_TOO_LONG             :constant Long :=    1109 ;
IDS_FILE_SAVEAS_DIALOG_NOTE        :constant Long :=    1110 ;

IDS_FILE_DRIVE_DISK_CHANGE         :constant Long :=    1120 ;
IDS_FILE_DRIVE_NOT_READY           :constant Long :=    1122 ;
IDS_FILE_DRIVE_LOCKED              :constant Long :=    1123 ;
IDS_FILE_DRIVE_NO_SECTOR           :constant Long :=    1124 ;
IDS_FILE_DRIVE_SOME_ERROR          :constant Long :=    1125 ;
IDS_FILE_DRIVE_INVALID             :constant Long :=    1126 ;
IDS_FILE_INSERT_DISK_NOTE          :constant Long :=    1127 ;
IDS_FILE_OK_WHEN_READY             :constant Long :=    1128 ;

   --********************************************************************--
   --                                                                    --
   --                     F O N T    D I A L O G                         --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- Font Dialog Creation Structure                                     --
   --********************************************************************--
subtype FIXED is USHORT;
   type FONTDLG is record    -- fntdlg --
    cbSize           :  ULONG   ;  -- sizeof(FONTDLG)
    hpsScreen        :  HPS     ;  -- Screen presentation space
    hpsPrinter       :  HPS     ;  -- Printer presentation space
    pszTitle         :  PSZ     ;  -- Application supplied title
    pszPreview       :  PSZ     ;  -- String to print in preview wndw
    pszPtSizeList    :  PSZ     ;  -- Application provided size list
    pfnDlgProc       :  PFNWP   ;  -- Dialog subclass procedure
    pszFamilyname    :  PSZ     ;  -- Family name of font
    fxPointSize      :  FIXED   ;  -- Point size the user selected
    fl               :  ULONG   ;  -- FNTS_* flags - dialog styles
    flFlags          :  ULONG   ;  -- FNTF_* state flags
    flType           :  ULONG   ;  -- Font type option bits
    flTypeMask       :  ULONG   ;  -- Mask of which font types to use
    flStyle          :  ULONG   ;  -- The selected style bits
    flStyleMask      :  ULONG   ;  -- Mask of which style bits to use
    clrFore          :  LONG    ;  -- Selected foreground color
    clrBack          :  LONG    ;  -- Selected background color
    ulUser           :  ULONG   ;  -- Blank field for application
    lReturn          :  LONG    ;  -- Return Value of the Dialog
    lSRC             :  LONG    ;  -- System return code.
    lEmHeight        :  LONG    ;  -- Em height of the current font
    lXHeight         :  LONG    ;  -- X height of the current font
    lExternalLeading :  LONG    ;  -- External Leading of font
    hMod             :  HMODULE ;  -- Module to load custom template
    f_Attrs          :  FATTRS  ;  -- Font attribute structure
    sNominalPointSize:  SHORT   ;  -- Nominal Point Size of font
    usWeight         :  USHORT  ;  -- The boldness of the font
    usWidth          :  USHORT  ;  -- The width of the font
    x                :  SHORT   ;  -- X coordinate of the dialog
    y                :  SHORT   ;  -- Y coordinate of the dialog
    usDlgId          :  USHORT  ;  -- ID of a custom dialog template
    usFamilyBufLen   :  USHORT  ;  -- Length of family buffer provided
    usReserved       :  USHORT  ;  -- reserved
   end record;
   type PFONTDLG is access all FONTDLG;

   --********************************************************************--
   -- Font Dialog Style Flags                                            --
   --********************************************************************--
 FNTS_CENTER             :constant Long := 16#00000001#; -- Center in owner dialog
 FNTS_CUSTOM             :constant Long := 16#00000002#; -- Use custom template
 FNTS_OWNERDRAWPREVIEW   :constant Long := 16#00000004#; -- Allow app to draw
 FNTS_HELPBUTTON         :constant Long := 16#00000008#; -- Display Help button
 FNTS_APPLYBUTTON        :constant Long := 16#00000010#; -- Display Apply button
 FNTS_RESETBUTTON        :constant Long := 16#00000020#; -- Display Reset button
 FNTS_MODELESS           :constant Long := 16#00000040#; -- Make dialog modeless
 FNTS_INITFROMFATTRS     :constant Long := 16#00000080#; -- Initialize from FATTRs
 FNTS_BITMAPONLY         :constant Long := 16#00000100#; -- Only allow bitmap font
 FNTS_VECTORONLY         :constant Long := 16#00000200#; -- Only allow vector font
 FNTS_FIXEDWIDTHONLY     :constant Long := 16#00000400#; -- Only allow monospaced
 FNTS_PROPORTIONALONLY   :constant Long := 16#00000800#; -- Only proportional font
 FNTS_NOSYNTHESIZEDFONTS :constant Long := 16#00001000#; -- Don't synthesize fonts

   --********************************************************************--
   -- Font Dialog Flags                                                  --
   --********************************************************************--
 FNTF_NOVIEWSCREENFONTS       :constant Long :=  1;
 FNTF_NOVIEWPRINTERFONTS      :constant Long :=  2;
 FNTF_SCREENFONTSELECTED      :constant Long :=  4;
 FNTF_PRINTERFONTSELECTED     :constant Long :=  8;

   --********************************************************************--
   -- Color code definitions                                             --
   --********************************************************************--
 CLRC_FOREGROUND               :constant Long := 1;
 CLRC_BACKGROUND               :constant Long := 2;

   --********************************************************************--
   -- Filter List message string identifiers                             --
   --********************************************************************--
FNTI_BITMAPFONT          :constant Long :=   16#0001#;
FNTI_VECTORFONT          :constant Long :=   16#0002#;
FNTI_FIXEDWIDTHFONT      :constant Long :=   16#0004#;
FNTI_PROPORTIONALFONT    :constant Long :=   16#0008#;
FNTI_SYNTHESIZED         :constant Long :=   16#0010#;
FNTI_DEFAULTLIST         :constant Long :=   16#0020#;
FNTI_FAMILYNAME          :constant Long :=   16#0100#;
FNTI_STYLENAME           :constant Long :=   16#0200#;
FNTI_POINTSIZE           :constant Long :=   16#0400#;

   --********************************************************************--
   -- Error Return Codes from dialog (self defining)                     --
   --********************************************************************--
 FNTS_SUCCESSFUL             :constant Long :=   0 ;
 FNTS_ERR_INVALID_DIALOG     :constant Long :=   3 ;
 FNTS_ERR_ALLOC_SHARED_MEM   :constant Long :=   4 ;
 FNTS_ERR_INVALID_PARM       :constant Long :=   5 ;
 FNTS_ERR_OUT_OF_MEMORY      :constant Long :=   7 ;
 FNTS_ERR_INVALID_VERSION    :constant Long :=  10 ;
 FNTS_ERR_DIALOG_LOAD_ERROR  :constant Long :=  12 ;

   --********************************************************************--
   -- Font Dialog Messages                                               --
   --********************************************************************--
FNTM_FACENAMECHANGED     :constant ULong :=  WM_USER+50; -- mp1 := PSZ pszFacename
FNTM_POINTSIZECHANGED    :constant ULong :=  WM_USER+51; -- mp1 := PSZ pszPointSize;
                                                        -- mp2 := FIXED fxPointSize
FNTM_STYLECHANGED        :constant ULong :=  WM_USER+52; -- mp1 := PSTYLECHANGE pstyc
FNTM_COLORCHANGED        :constant ULong :=  WM_USER+53; -- mp1 := LONG clr
                                                        -- mp2 := USHORT codeClr
FNTM_UPDATEPREVIEW       :constant ULong :=  WM_USER+54; -- mp1 := HWND hWndPreview
FNTM_FILTERLIST          :constant ULong :=  WM_USER+55; -- mp1 := PSZ pszFacename
                                                         -- mp2 := USHORT usStrStyle --
                                                         -- mr:=1(Add);0(Dont)--

   --********************************************************************--
   -- Stylechange message parameter structure                            --
   --********************************************************************--
   type STYLECHANGE is record    -- stylc --
    usWeight      :  USHORT      ;
    usWeightOld   :  USHORT      ;
    usWidth       :  USHORT      ;
    usWidthOld    :  USHORT      ;
    flType        :  ULONG       ;
    flTypeOld     :  ULONG       ;
    flTypeMask    :  ULONG       ;
    flTypeMaskOld :  ULONG       ;
    flStyle       :  ULONG       ;
    flStyleOld    :  ULONG       ;
    flStyleMask   :  ULONG       ;
    flStyleMaskOld:  ULONG       ;
   end record ;
   type PSTYLECHANGE is access all STYLECHANGE;

   --********************************************************************--
   -- Font Dialog Function Prototypes                                    --
   --********************************************************************--
function WinFontDlg(hwndP:HWND     ;
                    hwndO:HWND     ;
                    pfntd:PFONTDLG ) return  HWND    ;
pragma Import(c,WinFontDlg, Link_name=>"_WinFontDlg");

function WinDefFontDlgProc(h_wnd:HWND   ;
                           msg  :ULONG  ;
                           mp1  :MPARAM ;
                           mp2  :MPARAM ) return  MRESULT ;
pragma Import(c,WinDefFontDlgProc, Link_name=>"_WinDefFontDlgProc");

   --********************************************************************--
   -- font dialog and control id's                                       --
   --********************************************************************--
DID_FONT_DIALOG            :constant ULong :=  300 ;
DID_NAME                   :constant ULong :=  301 ;
DID_STYLE                  :constant ULong :=  302 ;
DID_DISPLAY_FILTER         :constant ULong :=  303 ;
DID_PRINTER_FILTER         :constant ULong :=  304 ;
DID_SIZE                   :constant ULong :=  305 ;
DID_SAMPLE                 :constant ULong :=  306 ;
DID_OUTLINE                :constant ULong :=  307 ;
DID_UNDERSCORE             :constant ULong :=  308 ;
DID_STRIKEOUT              :constant ULong :=  309 ;
DID_HELP_BUTTON            :constant ULong :=  310 ;
DID_APPLY_BUTTON           :constant ULong :=  311 ;
DID_RESET_BUTTON           :constant ULong :=  312 ;
DID_OK_BUTTON              :constant ULong :=  DID_OK ;
DID_CANCEL_BUTTON          :constant ULong :=  DID_CANCEL ;
DID_NAME_PREFIX            :constant ULong :=  313 ;
DID_STYLE_PREFIX           :constant ULong :=  314 ;
DID_SIZE_PREFIX            :constant ULong :=  315 ;
DID_SAMPLE_GROUPBOX        :constant ULong :=  316 ;
DID_EMPHASIS_GROUPBOX      :constant ULong :=  317 ;
DID_FONT_ISO_SUPPORT       :constant ULong :=  318 ;
DID_FONT_ISO_UNTESTED      :constant ULong :=  319 ;

   --********************************************************************--
   -- Stringtable id's                                                   --
   --********************************************************************--
IDS_FONT_SAMPLE            :constant Long :=    350 ;
IDS_FONT_BLANK             :constant Long :=    351 ;
IDS_FONT_KEY_0             :constant Long :=    352 ;
IDS_FONT_KEY_9             :constant Long :=    353 ;
IDS_FONT_KEY_SEP           :constant Long :=    354 ;
IDS_FONT_DISP_ONLY         :constant Long :=    355 ;
IDS_FONT_PRINTER_ONLY      :constant Long :=    356 ;
IDS_FONT_COMBINED          :constant Long :=    357 ;
IDS_FONT_WEIGHT1           :constant Long :=    358 ;
IDS_FONT_WEIGHT2           :constant Long :=    359 ;
IDS_FONT_WEIGHT3           :constant Long :=    360 ;
IDS_FONT_WEIGHT4           :constant Long :=    361 ;
IDS_FONT_WEIGHT5           :constant Long :=    362 ;
IDS_FONT_WEIGHT6           :constant Long :=    363 ;
IDS_FONT_WEIGHT7           :constant Long :=    364 ;
IDS_FONT_WEIGHT8           :constant Long :=    365 ;
IDS_FONT_WEIGHT9           :constant Long :=    366 ;
IDS_FONT_WIDTH1            :constant Long :=    367 ;
IDS_FONT_WIDTH2            :constant Long :=    368 ;
IDS_FONT_WIDTH3            :constant Long :=    369 ;
IDS_FONT_WIDTH4            :constant Long :=    370 ;
IDS_FONT_WIDTH5            :constant Long :=    371 ;
IDS_FONT_WIDTH6            :constant Long :=    372 ;
IDS_FONT_WIDTH7            :constant Long :=    373 ;
IDS_FONT_WIDTH8            :constant Long :=    374 ;
IDS_FONT_WIDTH9            :constant Long :=    375 ;
IDS_FONT_OPTION0           :constant Long :=    376 ;
IDS_FONT_OPTION1           :constant Long :=    377 ;
IDS_FONT_OPTION2           :constant Long :=    378 ;
IDS_FONT_OPTION3           :constant Long :=    379 ;
IDS_FONT_POINT_SIZE_LIST   :constant Long :=    380 ;

   --********************************************************************--
   --                                                                    --
   --                          S P I N    B U T T O N                    --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- SPINBUTTON Creation Flags                                          --
   --********************************************************************--

   --********************************************************************--
   -- Character Acceptance                                               --
   --********************************************************************--
SPBS_ALLCHARACTERS   :constant Long :=  16#00000000#; -- Default: All chars accepted
SPBS_NUMERICONLY     :constant Long :=  16#00000001#; -- Only 0 - 9 accepted & VKeys
SPBS_READONLY        :constant Long :=  16#00000002#; -- No chars allowed in entryfld

   --********************************************************************--
   -- Type of Component                                                  --
   --********************************************************************--
SPBS_MASTER          :constant Long :=  16#00000010#;
SPBS_SERVANT         :constant Long :=  16#00000000#; -- Default  Servant

   --********************************************************************--
   -- Type of Justification                                              --
   --********************************************************************--
SPBS_JUSTDEFAULT    :constant Long :=  16#00000000#; -- Default  Same as Left
SPBS_JUSTLEFT       :constant Long :=  16#00000008#;
SPBS_JUSTRIGHT      :constant Long :=  16#00000004#;
SPBS_JUSTCENTER     :constant Long :=  16#0000000C#;

   --********************************************************************--
   -- Border or not                                                      --
   --********************************************************************--
SPBS_NOBORDER       :constant Long :=  16#00000020#; -- Borderless SpinField
                                      --  Default is to have a border.

   --********************************************************************--
   -- Fast spin or not                                                   --
   --********************************************************************--
SPBS_FASTSPIN       :constant Long :=  16#00000100#; -- Allow fast spinning.  Fast
                                                     -- spinning is performed by
                                                     -- skipping over numbers

   --********************************************************************--
   -- Pad numbers on front with 0's                                      --
   --********************************************************************--
SPBS_PADWITHZEROS    :constant Long := 16#00000080#; -- Pad the number with zeroes

   --********************************************************************--
   -- SPINBUTTON Messages                                                --
   --********************************************************************--

   --********************************************************************--
   -- Notification from Spinbutton to the application is sent in a       --
   -- WM_CONTROL message.                                                --
   --********************************************************************--
SPBN_UPARROW         :constant Long :=  16#20A#;    -- up arrow button was pressed
SPBN_DOWNARROW       :constant Long :=  16#20B#;    -- down arrow button was pressed
SPBN_ENDSPIN         :constant Long :=  16#20C#;    -- mouse button was released
SPBN_CHANGE          :constant Long :=  16#20D#;    -- spinfield text has changed
SPBN_SETFOCUS        :constant Long :=  16#20E#;    -- spinfield received focus
SPBN_KILLFOCUS       :constant Long :=  16#20F#;    -- spinfield lost focus

   --********************************************************************--
   -- Messages from application to Spinbutton                            --
   --********************************************************************--
SPBM_OVERRIDESETLIMITS   :constant Long :=  16#200#;  -- Set spinbutton limits without
                                                      -- resetting the current value
SPBM_QUERYLIMITS         :constant Long :=  16#201#;  -- Query limits set by
                                                      -- SPBM_SETLIMITS
SPBM_SETTEXTLIMIT        :constant Long :=  16#202#;  -- Max entryfield characters
SPBM_SPINUP              :constant Long :=  16#203#;  -- Tell entry field to spin up
SPBM_SPINDOWN            :constant Long :=  16#204#;  -- Tell entry field to spin down
SPBM_QUERYVALUE          :constant Long :=  16#205#;  -- Tell entry field to send
                                                      -- current value

   --********************************************************************--
   -- Query Flags                                                        --
   --********************************************************************--
SPBQ_UPDATEIFVALID    :constant Long :=    0 ;     -- Default
SPBQ_ALWAYSUPDATE     :constant Long :=    1 ;
SPBQ_DONOTUPDATE      :constant Long :=    3 ;

   --********************************************************************--
   -- Return value for Empty Field.                                      --
   --    If ptr too long; variable sent in query msg                     --
   --********************************************************************--
SPBM_SETARRAY            :constant Long :=  16#206#;  -- Change the data to spin
SPBM_SETLIMITS           :constant Long :=  16#207#;  -- Change the numeric Limits
SPBM_SETCURRENTVALUE     :constant Long :=  16#208#;  -- Change the current value
SPBM_SETMASTER           :constant Long :=  16#209#;  -- Tell entryfield who master is

   --********************************************************************--
   --                                                                    --
   --                D I R E C T   M A N I P U L A T I O N               --
   --                                                                    --
   --********************************************************************--

 PMERR_NOT_DRAGGING        :constant Long :=16#1f00#;
 PMERR_ALREADY_DRAGGING    :constant Long :=16#1f01#;

 MSGF_DRAG                 :constant Long :=16#0010#;  -- message filter identifier

 WM_DRAGFIRST              :constant Long :=16#0310#;
 WM_DRAGLAST               :constant Long :=16#032f#;

 DM_DROP                   :constant Long :=16#032f#;
 DM_DRAGOVER               :constant Long :=16#032e#;
 DM_DRAGLEAVE              :constant Long :=16#032d#;
 DM_DROPHELP               :constant Long :=16#032c#;
 DM_ENDCONVERSATION        :constant Long :=16#032b#;
 DM_PRINT                  :constant Long :=16#032a#;
 DM_RENDER                 :constant Long :=16#0329#;
 DM_RENDERCOMPLETE         :constant Long :=16#0328#;
 DM_RENDERPREPARE          :constant Long :=16#0327#;
 DM_DRAGFILECOMPLETE       :constant Long :=16#0326#;
 DM_EMPHASIZETARGET        :constant Long :=16#0325#;
 DM_DRAGERROR              :constant Long :=16#0324#;
 DM_FILERENDERED           :constant Long :=16#0323#;
 DM_RENDERFILE             :constant Long :=16#0322#;
 DM_DRAGOVERNOTIFY         :constant Long :=16#0321#;
 DM_PRINTOBJECT            :constant Long :=16#0320#;
 DM_DISCARDOBJECT          :constant Long :=16#031f#;

  DRT_ASM          :constant String := "Assembler Code" ; -- drag type constants
  DRT_BASIC        :constant String := "BASIC Code" ;
  DRT_BINDATA      :constant String := "Binary Data";
  DRT_BITMAP       :constant String := "Bitmap" ;
  DRT_C            :constant String := "C Code" ;
  DRT_COBOL        :constant String := "COBOL Code" ;
  DRT_DLL          :constant String := "Dynamic Link Library" ;
  DRT_DOSCMD       :constant String := "DOS Command File" ;
  DRT_EXE          :constant String := "Executable" ;
  DRT_FORTRAN      :constant String := "FORTRAN Code" ;
  DRT_ICON         :constant String := "Icon"  ;
  DRT_LIB          :constant String := "Library"  ;
  DRT_METAFILE     :constant String := "Metafile" ;
  DRT_OS2CMD       :constant String := "OS/2 Command File" ;
  DRT_PASCAL       :constant String := "Pascal Code"  ;
  DRT_RESOURCE     :constant String := "Resource File" ;
  DRT_TEXT         :constant String := "Plain Text" ;
  DRT_UNKNOWN      :constant String := "Unknown" ;

DOR_NODROP              :constant Long :=   16#0000#; -- DM_DRAGOVER response codes --
DOR_DROP                :constant Long :=   16#0001#;
DOR_NODROPOP            :constant Long :=   16#0002#;
DOR_NEVERDROP           :constant Long :=   16#0003#;

DO_COPYABLE             :constant Long :=   16#0001#; -- supported operation flags  --
DO_MOVEABLE             :constant Long :=   16#0002#;
DO_LINKABLE             :constant Long :=   16#0004#;

DC_OPEN                 :constant Long :=   16#0001#; -- source control flags       --
DC_REF                  :constant Long :=   16#0002#;
DC_GROUP                :constant Long :=   16#0004#;
DC_CONTAINER            :constant Long :=   16#0008#;
DC_PREPARE              :constant Long :=   16#0010#;
DC_REMOVEABLEMEDIA      :constant Long :=   16#0020#;

DO_DEFAULT              :constant Long :=   16#BFFE#; -- Default operation          --
DO_UNKNOWN              :constant Long :=   16#BFFF#; -- Unknown operation          --
DO_COPY                 :constant Long :=   16#0010#;
DO_MOVE                 :constant Long :=   16#0020#;
DO_LINK                 :constant Long :=   16#0018#;
DO_CREATE               :constant Long :=   16#0040#;

DMFL_TARGETSUCCESSFUL   :constant Long := 16#0001#; -- transfer reply flags       --
DMFL_TARGETFAIL         :constant Long := 16#0002#;
DMFL_NATIVERENDER       :constant Long := 16#0004#;
DMFL_RENDERRETRY        :constant Long := 16#0008#;
DMFL_RENDEROK           :constant Long := 16#0010#;
DMFL_RENDERFAIL         :constant Long := 16#0020#;

DRG_ICON           :constant Long :=   16#00000001#;  -- drag image manipulation
DRG_BITMAP         :constant Long :=   16#00000002#;  --   flags
DRG_POLYGON        :constant Long :=   16#00000004#;
DRG_STRETCH        :constant Long :=   16#00000008#;
DRG_TRANSPARENT    :constant Long :=   16#00000010#;
DRG_CLOSED         :constant Long :=   16#00000020#;

DME_IGNOREABORT        :constant Long :=    1 ;      -- DM_DRAGERROR return values
DME_IGNORECONTINUE     :constant Long :=    2 ;
DME_REPLACE            :constant Long :=    3 ;
DME_RETRY              :constant Long :=    4 ;

DF_MOVE                 :constant Long :=   16#0001#; -- DM_DRAGFILECOMPLETE flags
DF_SOURCE               :constant Long :=   16#0002#;
DF_SUCCESSFUL           :constant Long :=   16#0004#;

 DRR_SOURCE           :constant Long :=    1;
 DRR_TARGET           :constant Long :=    2;
 DRR_ABORT            :constant Long :=    3;

 DFF_MOVE             :constant Long :=     1;       -- DM_DRAGERROR operation IDs --
 DFF_COPY             :constant Long :=     2;
 DFF_DELETE           :constant Long :=     3;


subtype HSTR is LHANDLE ;  -- hstr --

   type DRAGITEM is record    -- ditem --
    hwndItem         :  HWND    ;  -- conversation partner
    ulItemID         :  ULONG   ;  -- identifies item being dragged
    hstrType         :  HSTR    ;  -- type of item
    hstrRMF          :  HSTR    ;  -- rendering mechanism and format
    hstrContainerName:  HSTR    ;  -- name of source container
    hstrSourceName   :  HSTR    ;  -- name of item at source
    hstrTargetName   :  HSTR    ;  -- suggested name of item at dest
    cxOffset         :  SHORT   ;  -- x offset of the origin of the
                                   -- image from the mouse hotspot
    cyOffset         :  SHORT   ;  -- y offset of the origin of the
                                   -- image from the mouse hotspot
    fsControl        :  USHORT  ;  -- source item control flags
    fsSupportedOps   :  USHORT  ;  -- ops supported by source
   end record;
   type PDRAGITEM  is access all DRAGITEM;

   type DRAGINFO is record    -- dinfo --
    cbDraginfo :  ULONG    ;  -- Size of DRAGINFO and DRAGITEMs
    cbDragitem :  USHORT   ;  -- size of DRAGITEM
    usOperation:  USHORT   ;  -- current drag operation
    hwndSource :  HWND     ;  -- window handle of source
    xDrop      :  SHORT    ;  -- x coordinate of drop position
    yDrop      :  SHORT    ;  -- y coordinate of drop position
    cditem     :  USHORT   ;  -- count of DRAGITEMs
    usReserved :  USHORT   ;  -- reserved for future use
   end record;
   type PDRAGINFO is access all DRAGINFO;

   type DRAGIMAGE is record    -- dimg --
    cb         :  USHORT  ;  -- size control block
    cptl       :  USHORT  ;  -- count of pts; if DRG_POLYGON
    hImage     :  LHANDLE ;  -- image handle passed to DrgDrag
    sizlStretch:  SIZEL   ;  -- size to strecth ico or bmp to
    fl         :  ULONG   ;  -- flags passed to DrgDrag
    cxOffset   :  SHORT   ;  -- x offset of the origin of the
                             -- image from the mouse hotspot
    cyOffset   :  SHORT   ;  -- y offset of the origin of the
                             -- image from the mouse hotspot
   end record;
   type PDRAGIMAGE is access all DRAGIMAGE;

   type DRAGTRANSFER is record    -- dxfer --
    cb              :  ULONG      ;   -- size of control block
    hwndClient      :  HWND       ;   -- handle of target
    pditem          :  PDRAGITEM  ;   -- DRAGITEM being transferred
    hstrSelectedRMF :  HSTR       ;   -- rendering mech & fmt of choice
    hstrRenderToName:  HSTR       ;   -- name source will use
    ulTargetInfo    :  ULONG      ;   -- reserved for target's use
    usOperation     :  USHORT     ;   -- operation being performed
    fsReply         :  USHORT     ;   -- reply flags
   end record ;
   type PDRAGTRANSFER is access all DRAGTRANSFER;

   type RENDERFILE   is record  -- rndf --
    hwndDragFiles:  HWND   ;  -- conversation window
    hstrSource   :  HSTR   ;  -- handle to source file name
    hstrTarget   :  HSTR   ;  -- handle to target file name
    fMove        :  USHORT ;  -- 1 - move; 0 - copy
    usRsvd       :  USHORT ;  -- reserved
   end record  ;
   type PRENDERFILE is access all RENDERFILE;

function DrgAcceptDroppedFiles(h_wnd      :HWND  ;
                               pszPath    :PSZ   ;
                               pszTypes   :PSZ   ;
                               ulDefaultOp:ULONG ;
                               ulRsvd     :ULONG ) return  BOOL ;
pragma Import(c,DrgAcceptDroppedFiles, Link_name=>"_DrgAcceptDroppedFiles");

function DrgAllocDraginfo(cditem:ULONG ) return  PDRAGINFO ;
pragma Import(c,DrgAllocDraginfo, Link_name=>"_DrgAllocDraginfo");

function DrgAllocDragtransfer(cdxfer:ULONG ) return PDRAGTRANSFER ;
pragma Import(c,DrgAllocDragtransfer, Link_name=>"_DrgAllocDragtransfer");

function DrgDrag(hwndSource :HWND       ;
                 pdinfo     :PDRAGINFO  ;
                 pdimg      :PDRAGIMAGE ;
                 cdimg      :ULONG      ;
                 vkTerminate:LONG       ;
                 pRsvd      :PVOID      ) return  HWND      ;
pragma Import(c,DrgDrag, Link_name=>"_DrgDrag");
subtype HPOINTER is PVOID ;
function DrgDragFiles(h_wnd        :HWND     ;
                      apszFiles    :APSZ     ;
                      apszTypes    :APSZ     ;
                      apszTargets  :APSZ     ;
                      cFiles       :ULONG    ;
                      hptrDrag     :HPOINTER ;
                      vkTerm       :ULONG    ;
                      fSourceRender:BOOL     ;
                      ulRsvd       :ULONG    ) return   BOOL      ;
pragma Import(c,DrgDragFiles, Link_name=>"_DrgDragFiles");

function DrgPostTransferMsg(h_wnd :HWND          ;
                            msg   :ULONG         ;
                            pdxfer:PDRAGTRANSFER ;
                            fl    :ULONG         ;
                            ulRsvd:ULONG         ;
                            fRetry:BOOL          ) return  BOOL      ;
pragma Import(c,DrgPostTransferMsg, Link_name=>"_DrgPostTransferMsg");

function DrgQueryDragitem(pdinfo  :PDRAGINFO ;
                          cbBuffer:ULONG     ;
                          pditem  :PDRAGITEM ;
                          iItem   :ULONG     )  return BOOL      ;
pragma Import(c,DrgQueryDragitem, Link_name=>"_DrgQueryDragitem");

function DrgQueryDragitemCount(pdinfo:PDRAGINFO ) return  ULONG     ;
pragma Import(c,DrgQueryDragitemCount, Link_name=>"_DrgQueryDragitemCount");

function DrgQueryDragitemPtr(pdinfo:PDRAGINFO ;
                             i     :ULONG     ) return   PDRAGITEM ;
pragma Import(c,DrgQueryDragitemPtr, Link_name=>"_DrgQueryDragitemPtr");

function DrgQueryNativeRMF(pditem  :PDRAGITEM ;
                           cbBuffer:ULONG     ;
                           pBuffer :PCHAR     ) return  BOOL      ;
pragma Import(c,DrgQueryNativeRMF, Link_name=>"_DrgQueryNativeRMF");

function DrgQueryNativeRMFLen(pditem:PDRAGITEM ) return  ULONG     ;
pragma Import(c,DrgQueryNativeRMFLen, Link_name=>"_DrgQueryNativeRMFLen");

function DrgQueryStrName(h_str   :HSTR  ;
                         cbBuffer:ULONG ;
                         pBuffer :PSZ   ) return  ULONG     ;
pragma Import(c,DrgQueryStrName, Link_name=>"_DrgQueryStrName");

function DrgQueryStrNameLen(h_str:HSTR ) return  ULONG     ;
pragma Import(c,DrgQueryStrNameLen, Link_name=>"_DrgQueryStrNameLen");

function DrgQueryTrueType(pditem  :PDRAGITEM ;
                          cbBuffer:ULONG     ;
                          pBuffer :PSZ       ) return  BOOL      ;
pragma Import(c,DrgQueryTrueType, Link_name=>"_DrgQueryTrueType");

function DrgQueryTrueTypeLen(pditem:PDRAGITEM ) return  ULONG     ;
pragma Import(c,DrgQueryTrueTypeLen, Link_name=>"_DrgQueryTrueTypeLen");

function DrgSendTransferMsg(h_wnd:HWND   ;
                            msg  :ULONG  ;
                            mp1  :MPARAM ;
                            mp2  :MPARAM ) return  MRESULT   ;
pragma Import(c,DrgSendTransferMsg, Link_name=>"_DrgSendTransferMsg");

function DrgSetDragitem(pdinfo  :PDRAGINFO ;
                        pditem  :PDRAGITEM ;
                        cbBuffer:ULONG     ;
                        iItem   :ULONG     ) return  BOOL      ;
pragma Import(c,DrgSetDragitem, Link_name=>"_DrgSetDragitem");

function DrgSetDragImage(pdinfo:PDRAGINFO  ;
                         pdimg :PDRAGIMAGE ;
                         cdimg :ULONG      ;
                         pRsvd :PVOID      ) return  BOOL      ;
pragma Import(c,DrgSetDragImage, Link_name=>"_DrgSetDragImage");

function DrgVerifyTypeSet(pditem  :PDRAGITEM ;
                          pszType :PSZ       ;
                          cbMatch :ULONG     ;
                          pszMatch:PSZ       ) return  BOOL      ;
pragma Import(c,DrgVerifyTypeSet, Link_name=>"_DrgVerifyTypeSet");

function DrgAccessDraginfo(pdinfo:PDRAGINFO ) return  BOOL      ;
pragma Import(c,DrgAccessDraginfo, Link_name=>"_DrgAccessDraginfo");

function DrgAddStrHandle(p_sz:PSZ ) return  HSTR      ;
pragma Import(c,DrgAddStrHandle, Link_name=>"_DrgAddStrHandle");

function DrgDeleteDraginfoStrHandles(pdinfo:PDRAGINFO ) return  BOOL ;
pragma Import(c,DrgDeleteDraginfoStrHandles, Link_name=>"_DrgDeleteDraginfoStrHandles");

function DrgDeleteStrHandle(h_str:HSTR ) return  BOOL      ;
pragma Import(c,DrgDeleteStrHandle, Link_name=>"_DrgDeleteStrHandle");

function DrgFreeDraginfo(pdinfo:PDRAGINFO ) return  BOOL      ;
pragma Import(c,DrgFreeDraginfo, Link_name=>"_DrgFreeDraginfo");

function DrgFreeDragtransfer(pdxfer:PDRAGTRANSFER ) return   BOOL      ;
pragma Import(c,DrgFreeDragtransfer, Link_name=>"_DrgFreeDragtransfer");

function DrgGetPS(h_wnd:HWND ) return   HPS       ;
pragma Import(c,DrgGetPS, Link_name=>"_DrgGetPS");

function DrgPushDraginfo(pdinfo  :PDRAGINFO ;
                         hwndDest:HWND      ) return   BOOL      ;
pragma Import(c,DrgPushDraginfo, Link_name=>"_DrgPushDraginfo");

function DrgReleasePS(h_ps:HPS ) return   BOOL      ;
pragma Import(c,DrgReleasePS, Link_name=>"_DrgReleasePS");

function DrgSetDragPointer(pdinfo:PDRAGINFO ;
                           hptr  :HPOINTER  ) return   BOOL      ;
pragma Import(c,DrgSetDragPointer, Link_name=>"_DrgSetDragPointer");

function DrgVerifyNativeRMF(pditem:PDRAGITEM ;
                            pszRMF:PSZ       ) return   BOOL      ;
pragma Import(c,DrgVerifyNativeRMF, Link_name=>"_DrgVerifyNativeRMF");

function DrgVerifyRMF(pditem :PDRAGITEM ;
                      pszMech:PSZ       ;
                      pszFmt :PSZ       ) return   BOOL      ;
pragma Import(c,DrgVerifyRMF, Link_name=>"_DrgVerifyRMF");

function DrgVerifyTrueType(pditem :PDRAGITEM ;
                           pszType:PSZ       ) return   BOOL      ;
pragma Import(c,DrgVerifyTrueType, Link_name=>"_DrgVerifyTrueType");

function DrgVerifyType(pditem :PDRAGITEM ;
                       pszType:PSZ       ) return   BOOL      ;
pragma Import(c,DrgVerifyType, Link_name=>"_DrgVerifyType");
   --********************************************************************--
   --                                                                    --
   --                         C O N T A I N E R                          --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- Error constants                                                    --
   --********************************************************************--
PMERR_NOFILTERED_ITEMS             :constant Long := 16#1f02#;
PMERR_COMPARISON_FAILED            :constant Long := 16#1f03#;
PMERR_RECORD_CURRENTLY_INSERTED    :constant Long := 16#1f04#;
PMERR_FI_CURRENTLY_INSERTED        :constant Long := 16#1f05#;

   --********************************************************************--
   -- Container control styles.                                          --
   --********************************************************************--
 CCS_EXTENDSEL              :constant Long :=  16#00000001#;
 CCS_MULTIPLESEL            :constant Long :=  16#00000002#;
 CCS_SINGLESEL              :constant Long :=  16#00000004#;
 CCS_AUTOPOSITION           :constant Long :=  16#00000008#;
 CCS_VERIFYPOINTERS         :constant Long :=  16#00000010#;
 CCS_READONLY               :constant Long :=  16#00000020#;
 CCS_MINIRECORDCORE         :constant Long :=  16#00000040#;

   --********************************************************************--
   -- view identifiers           (flWindowAttr)                          --
   --********************************************************************--
 CV_TEXT     :constant Long :=  16#00000001#; -- text view
 CV_NAME     :constant Long :=  16#00000002#; -- name view
 CV_ICON     :constant Long :=  16#00000004#; -- icon view
 CV_DETAIL   :constant Long :=  16#00000008#; -- detail view
 CV_FLOW     :constant Long :=  16#00000010#; -- flow items
 CV_MINI     :constant Long :=  16#00000020#; -- use mini icon
 CV_TREE     :constant Long :=  16#00000040#; -- tree view

   --********************************************************************--
   -- Container Attributes        (flWindowAttr)                         --
   --********************************************************************--
 CA_CONTAINERTITLE          :constant Long := 16#00000200#;
 CA_TITLESEPARATOR          :constant Long := 16#00000400#;
 CA_TITLELEFT               :constant Long := 16#00000800#;
 CA_TITLERIGHT              :constant Long := 16#00001000#;
 CA_TITLECENTER             :constant Long := 16#00002000#;
 CA_OWNERDRAW               :constant Long := 16#00004000#;
 CA_DETAILSVIEWTITLES       :constant Long := 16#00008000#;
 CA_ORDEREDTARGETEMPH       :constant Long := 16#00010000#;
 CA_DRAWBITMAP              :constant Long := 16#00020000#;
 CA_DRAWICON                :constant Long := 16#00040000#;
 CA_TITLEREADONLY           :constant Long := 16#00080000#;
 CA_OWNERPAINTBACKGROUND    :constant Long := 16#00100000#;
 CA_MIXEDTARGETEMPH         :constant Long := 16#00200000#;
 CA_TREELINE                :constant Long := 16#00400000#;

   --********************************************************************--
   -- child window IDs                                                   --
   --********************************************************************--
 CID_LEFTCOLTITLEWND      :constant Long :=  16#7FF0#;   -- column title (left)
 CID_RIGHTCOLTITLEWND     :constant Long :=  16#7FF1#;   -- right column title
 CID_BLANKBOX             :constant Long :=  16#7FF2#;   -- blank box at bottom right
 CID_HSCROLL              :constant Long :=  16#7FF3#;   -- horizontal scroll bar
 CID_RIGHTHSCROLL         :constant Long :=  16#7FF4#;   -- right horz scroll bar
 CID_CNRTITLEWND          :constant Long :=  16#7FF5#;   -- Container title window
 CID_LEFTDVWND            :constant Long :=  16#7FF7#;   -- Left Details View window
 CID_RIGHTDVWND           :constant Long :=  16#7FF8#;   -- Right Details View window
 CID_VSCROLL              :constant Long :=  16#7FF9#;   -- vertical scroll bar
 CID_MLE                  :constant Long :=  16#7FFA#;   -- MLE window for direct edit

   --********************************************************************--
   -- Bitmap descriptor array element.                                   --
   --********************************************************************--
   type TREEITEMDESC is record    -- treend --
    hbmExpanded  :  HBITMAP       ;
    hbmCollapsed :  HBITMAP       ;
    hptrExpanded :  HPOINTER      ;
    hptrCollapsed:  HPOINTER      ;
   end record ;
   type PTREEITEMDESC is access all TREEITEMDESC;

   --********************************************************************--
   -- Field Info data structure; attribute and data types; CV_DETAIL     --
   --********************************************************************--
   type FIELDINFO  ;   -- fldinfo fictive --
   type PFIELDINFO is access all FIELDINFO;
   type FIELDINFO  is record    -- fldinfo --
    cb            :  ULONG      ;  -- size of FIELDINFO struct
    flData        :  ULONG      ;  -- attributes of field's data
    flTitle       :  ULONG      ;  -- attributes of field's title
    pTitleData    :  PVOID      ;  -- title data (when others is string)
                                   -- If CFT_BITMAP; must be HBITMAP
    offStruct     :  ULONG      ;  -- offset from RECORDCORE to data
    pUserData     :  PVOID      ;  -- pointer to user data
    pNextFieldInfo:  PFIELDINFO ;  -- pointer to next linked
                                   -- FIELDINFO structure
    cxWidth       :  ULONG      ;  -- width of field in pels
   end record ;

   --********************************************************************--
   -- RECORDCORE data structure ; attribute values                       --
   --********************************************************************--
   type RECORDCORE ;   -- recc --
   type PRECORDCORE is access all RECORDCORE;
   type RECORDCORE  is record   -- recc --
    cb             :  ULONG         ;
    flRecordAttr   :  ULONG         ;  -- record attributes
    ptlIcon        :  POINTL        ;  -- Position of CV_ICON item
    preccNextRecord:  PRECORDCORE   ;  -- ptr to next record
    pszIcon        :  PSZ           ;  -- Text for CV_ICON view
    hptrIcon       :  HPOINTER      ;  -- Icon to display for ~CV_MINI
    hptrMiniIcon   :  HPOINTER      ;  -- Icon to display for CV_MINI
    hbmBitmap      :  HBITMAP       ;  -- Bitmap to display for ~CV_MINI
    hbmMiniBitmap  :  HBITMAP       ;  -- Bitmap to display for CV_MINI
    p_TreeItemDesc :  PTREEITEMDESC ;  -- Icons for the tree view
    pszText        :  PSZ           ;  -- Text for CV_TEXT view
    pszName        :  PSZ           ;  -- Text for CV_NAME view
    pszTree        :  PSZ           ;  -- Text for CV_TREE view
   end record  ;

   --********************************************************************--
   -- MINIRECORDCORE data structure; attribute values                    --
   --********************************************************************--
   type MINIRECORDCORE ;   -- minirec --
   type PMINIRECORDCORE is access all MINIRECORDCORE;
   type MINIRECORDCORE  is record   -- minirec --
    cb             :  ULONG           ;
    flRecordAttr   :  ULONG           ; -- record attributes
    ptlIcon        :  POINTL          ; -- Position of CV_ICON item
    preccNextRecord:  PMINIRECORDCORE ; --ptr to next record --
    pszIcon        :  PSZ             ; -- Text for CV_ICON view
    hptrIcon       :  HPOINTER        ; -- Icon to display for ~CV_MINI
   end record ;

   --********************************************************************--
   -- CNRINFO data structure; describes the container control.           --
   --********************************************************************--
   type CNRINFO is record    -- ccinfo --
    cb                 : ULONG       ;  -- size of CNRINFO struct
    pSortRecord        : PVOID       ;  -- ptr to sort function;
                                        -- RECORDCORE
    pFieldInfoLast     : PFIELDINFO  ;  -- pointer to last column in
                                        -- left pane of a split window.
    pFieldInfoObject   : PFIELDINFO  ;  -- Pointer to a column to
                                        -- represent an object.  This is
                                        -- the column which will receive
                                        -- IN-USE emphasis.
    pszCnrTitle        : PSZ         ;  -- text for container title. One
                                        --  string separated by line
                                        --  separators for multi-lines
    flWindowAttr       : ULONG       ;  -- container attrs - CV_*; CA_*
    ptlOrigin          : POINTL      ;  -- lower-left origin in virtual
                                        --  coordinates. CV_ICON view
    cDelta             : ULONG       ;  -- Application defined threshold
                                        --  or number of records from
                                        --  either end of the list.
    cRecords           : ULONG       ;  -- number of records in container
    slBitmapOrIcon     : SIZEL       ;  -- size of bitmap in pels
    slTreeBitmapOrIcon : SIZEL       ;  -- size of tree bitmaps in pels
    hbmExpanded        : HBITMAP     ;  -- bitmap  for tree node
    hbmCollapsed       : HBITMAP     ;  -- bitmap  for tree node
    hptrExpanded       : HPOINTER    ;  -- icon    for tree node
    hptrCollapsed      : HPOINTER    ;  -- icon    for tree node
    cyLineSpacing      : LONG        ;  -- space between two rows
    cxTreeIndent       : LONG        ;  -- indent for children
    cxTreeLine         : LONG        ;  -- thickness of the Tree Line
    cFields            : ULONG       ;  -- number of fields  in container
    xVertSplitbar      : LONG        ;  -- position relative to the
                                        --  container (CV_DETAIL) return; if
                                        --  16#FFFF# then unsplit
   end record; -- CNRINFO;
   type PCNRINFO is access all CNRINFO;

   type CDATE  is record   -- cdate --
    day  :  UCHAR   ;   -- current day
    month:  UCHAR   ;   -- current month
    year :  USHORT  ;   -- current year
   end record; -- CDATE;
   type PCDATE is access all CDATE;

   type CTIME  is record   -- ctime --
    hours     :  UCHAR   ;   -- current hour
    minutes   :  UCHAR   ;   -- current minute
    seconds   :  UCHAR   ;   -- current second
    ucReserved:  UCHAR   ;   -- reserved
   end record; -- CTIME;
   type PCTIME is access all CTIME;

   --********************************************************************--
   -- attribute and type values for flData and flTitle members of        --
   -- FIELDINFO; CFA_ (attributes) return; CFT_ (types)                         --
   --********************************************************************--
 CFA_LEFT              :constant Long := 16#00000001#; -- left align text
 CFA_RIGHT             :constant Long := 16#00000002#; -- right align text
 CFA_CENTER            :constant Long := 16#00000004#; -- center text
 CFA_TOP               :constant Long := 16#00000008#; -- top-align text
 CFA_VCENTER           :constant Long := 16#00000010#; -- vertically center text
 CFA_BOTTOM            :constant Long := 16#00000020#; -- bottom-align text
 CFA_INVISIBLE         :constant Long := 16#00000040#; -- Specify invisible column.
 CFA_BITMAPORICON      :constant Long := 16#00000100#; -- field title is bitmap
 CFA_SEPARATOR         :constant Long := 16#00000200#; -- vert sep; right of fld
 CFA_HORZSEPARATOR     :constant Long := 16#00000400#; -- horz sep; bottom of fld

 CFA_STRING            :constant Long := 16#00000800#; -- string of characters
 CFA_OWNER             :constant Long := 16#00001000#; -- ownerdraw field
 CFA_DATE              :constant Long := 16#00002000#; -- date structure
 CFA_TIME              :constant Long := 16#00004000#; -- time structure
 CFA_FIREADONLY        :constant Long := 16#00008000#; -- Column is read-only.
 CFA_FITITLEREADONLY   :constant Long := 16#00010000#; -- Column Title is read-only
 CFA_ULONG             :constant Long := 16#00020000#; -- Column is number format

   --********************************************************************--
   -- attribute values for flRecordAttr member of RECORDCORE             --
   --********************************************************************--
 CRA_SELECTED          :constant Long := 16#00000001#; -- record is selected
 CRA_TARGET            :constant Long := 16#00000002#; -- record has target emphasis
 CRA_CURSORED          :constant Long := 16#00000004#; -- cursor is on the record
 CRA_INUSE             :constant Long := 16#00000008#; -- record has in-use emphasis
 CRA_FILTERED          :constant Long := 16#00000010#; -- record has been filtered
 CRA_DROPONABLE        :constant Long := 16#00000020#; -- record can be dropped on
 CRA_RECORDREADONLY    :constant Long := 16#00000040#; -- record is read-only
 CRA_EXPANDED          :constant Long := 16#00000080#; -- record is expanded
 CRA_COLLAPSED         :constant Long := 16#00000100#; -- record is collapsed

   --********************************************************************--
   -- Container messages                                                 --
   --********************************************************************--
 CM_ALLOCDETAILFIELDINFO         :constant Long := 16#0330#;
 CM_ALLOCRECORD                  :constant Long := 16#0331#;
 CM_ARRANGE                      :constant Long := 16#0332#;
 CM_ERASERECORD                  :constant Long := 16#0333#;
 CM_FILTER                       :constant Long := 16#0334#;
 CM_FREEDETAILFIELDINFO          :constant Long := 16#0335#;
 CM_FREERECORD                   :constant Long := 16#0336#;
 CM_HORZSCROLLSPLITWINDOW        :constant Long := 16#0337#;
 CM_INSERTDETAILFIELDINFO        :constant Long := 16#0338#;
 CM_INSERTRECORD                 :constant Long := 16#0339#;
 CM_INVALIDATEDETAILFIELDINFO    :constant Long := 16#033a#;
 CM_INVALIDATERECORD             :constant Long := 16#033b#;
 CM_PAINTBACKGROUND              :constant Long := 16#033c#;
 CM_QUERYCNRINFO                 :constant Long := 16#033d#;
 CM_QUERYDETAILFIELDINFO         :constant Long := 16#033e#;
 CM_QUERYDRAGIMAGE               :constant Long := 16#033f#;
 CM_QUERYRECORD                  :constant Long := 16#0340#;
 CM_QUERYRECORDEMPHASIS          :constant Long := 16#0341#;
 CM_QUERYRECORDFROMRECT          :constant Long := 16#0342#;
 CM_QUERYRECORDRECT              :constant Long := 16#0343#;
 CM_QUERYVIEWPORTRECT            :constant Long := 16#0344#;
 CM_REMOVEDETAILFIELDINFO        :constant Long := 16#0345#;
 CM_REMOVERECORD                 :constant Long := 16#0346#;
 CM_SCROLLWINDOW                 :constant Long := 16#0347#;
 CM_SEARCHSTRING                 :constant Long := 16#0348#;
 CM_SETCNRINFO                   :constant Long := 16#0349#;
 CM_SETRECORDEMPHASIS            :constant Long := 16#034a#;
 CM_SORTRECORD                   :constant Long := 16#034b#;
 CM_OPENEDIT                     :constant Long := 16#034c#;
 CM_CLOSEEDIT                    :constant Long := 16#034d#;
 CM_COLLAPSETREE                 :constant Long := 16#034e#;
 CM_EXPANDTREE                   :constant Long := 16#034f#;
 CM_QUERYRECORDINFO              :constant Long := 16#0350#;

   --********************************************************************--
   -- Container notifications                                            --
   --********************************************************************--
 CN_DRAGAFTER                    :constant Long := 101;
 CN_DRAGLEAVE                    :constant Long := 102;
 CN_DRAGOVER                     :constant Long := 103;
 CN_DROP                         :constant Long := 104;
 CN_DROPHELP                     :constant Long := 105;
 CN_ENTER                        :constant Long := 106;
 CN_INITDRAG                     :constant Long := 107;
 CN_EMPHASIS                     :constant Long := 108;
 CN_KILLFOCUS                    :constant Long := 109;
 CN_SCROLL                       :constant Long := 110;
 CN_QUERYDELTA                   :constant Long := 111;
 CN_SETFOCUS                     :constant Long := 112;
 CN_REALLOCPSZ                   :constant Long := 113;
 CN_BEGINEDIT                    :constant Long := 114;
 CN_ENDEDIT                      :constant Long := 115;
 CN_COLLAPSETREE                 :constant Long := 116;
 CN_EXPANDTREE                   :constant Long := 117;
 CN_HELP                         :constant Long := 118;
 CN_CONTEXTMENU                  :constant Long := 119;

   --********************************************************************--
   --   Data Structures for Message Parameters                           --
   --********************************************************************--

   --********************************************************************--
   -- Container Direct Manipulation structures                           --
   --********************************************************************--
   type CNRDRAGINIT is record    -- cdrginit --
    hwndCnr:  HWND        ;  -- Container window handle
    pRecord:  PRECORDCORE ;  -- record under mouse ptr
    x      :  LONG        ;  -- x coordinate of mouse ptr
    y      :  LONG        ;  -- y coordinate of mouse ptr
    cx     :  LONG        ;  -- x offset from record
    cy     :  LONG        ;  -- y offset from record
   end record; -- CNRDRAGINIT;
   type PCNRDRAGINIT is access all CNRDRAGINIT;

   --********************************************************************--
   -- Data structure for CM_INSERTDETAILFIELDINFO                        --
   -- This structure is used by the application to specify the position  --
   -- of the FieldInfo structures they are inserting.                    --
   --********************************************************************--
   type FIELDINFOINSERT is record    -- fiins --
    cb                  :  ULONG       ; -- Size of structure.
    pFieldInfoOrder     :  PFIELDINFO  ; -- Specifies the order of the
                                         --             FieldInfo structures.
    fInvalidateFieldInfo:  ULONG       ; -- Invalidate on Insert.
    cFieldInfoInsert    :  ULONG       ; -- The number of FieldInfo
                                         --  structures to insert.
   end record; -- FIELDINFOINSERT;
   type PFIELDINFOINSERT is access all FIELDINFOINSERT;

   --********************************************************************--
   -- Data structure for CM_INSERTRECORD                                 --
   --********************************************************************--
   type RECORDINSERT is record    -- recins --
    cb               :  ULONG       ;
    pRecordOrder     :  PRECORDCORE ;
    pRecordParent    :  PRECORDCORE ;
    fInvalidateRecord:  ULONG       ;
    zOrder           :  ULONG       ;
    cRecordsInsert   :  ULONG       ;
   end record; -- RECORDINSERT;
   type PRECORDINSERT is access all RECORDINSERT;

   --********************************************************************--
   -- Data structure for CM_QUERYRECORDFROMRECT                          --
   --********************************************************************--
   type QUERYRECFROMRECT is record    -- qrecfrct --
    cb      :  ULONG   ;
    rect    :  RECTL   ;
    fsSearch:  ULONG   ;
   end record; -- QUERYRECFROMRECT;
   type PQUERYRECFROMRECT is access all QUERYRECFROMRECT;

   --********************************************************************--
   -- Data structure for CM_QUERYRECORDRECT                              --
   --********************************************************************--
   type QUERYRECORDRECT  is record   -- qrecrct --
    cb               :  ULONG        ;
    pRecord          :  PRECORDCORE  ;
    fRightSplitWindow:  ULONG        ;
    fsExtent         :  ULONG        ;
   end record; -- QUERYRECORDRECT;
   type PQUERYRECORDRECT is access all QUERYRECORDRECT;

   --********************************************************************--
   -- Data structure for CM_SEARCHSTRING                                 --
   --********************************************************************--
   type SEARCHSTRING is record    -- srchstr --
    cb             :  ULONG  ;
    pszSearch      :  PSZ    ;
    fsPrefix       :  ULONG  ;
    fsCaseSensitive:  ULONG  ;
    usView         :  ULONG  ;
   end record; -- SEARCHSTRING;
   type PSEARCHSTRING is access all SEARCHSTRING;

      --*****************************************************************--
      -- Data Structure for CN_DRAGLEAVE;CN_DRAGOVER;CN_DROP;CN_DROPHELP --
      --*****************************************************************--
      type CNRDRAGINFO is record    -- cdrginfo --
       p_DragInfo:  PDRAGINFO   ;
       pRecord  :  PRECORDCORE ;
      end record; -- CNRDRAGINFO;
      type PCNRDRAGINFO is access all CNRDRAGINFO;

   --********************************************************************--
   -- Data structure for CN_EMPHASIS                                     --
   --********************************************************************--
   type NOTIFYRECORDEMPHASIS is record    -- nrecemph --
    hwndCnr      :  HWND        ;
    pRecord      :  PRECORDCORE ;
    fEmphasisMask:  ULONG       ;
   end record; -- NOTIFYRECORDEMPHASIS;
   type PNOTIFYRECORDEMPHASIS is access all NOTIFYRECORDEMPHASIS;

   --********************************************************************--
   -- Data structure for CN_ENTER                                        --
   --********************************************************************--
   type NOTIFYRECORDENTER  is record   -- notrecen --
    hwndCnr:  HWND        ;
    fKey   :  ULONG       ;
    pRecord:  PRECORDCORE ;
   end record; -- NOTIFYRECORDENTER;
   type PNOTIFYRECORDENTER is access all NOTIFYRECORDENTER;

   --********************************************************************--
   -- Data structure for CN_QUERYDELTA                                   --
   --********************************************************************--
   type NOTIFYDELTA is record    -- notdlt --
    hwndCnr:  HWND        ;
    fDelta :  ULONG       ;
   end record; -- NOTIFYDELTA;
   type PNOTIFYDELTA is access all NOTIFYDELTA;

   --********************************************************************--
   -- Data structure for CN_SCROLL                                       --
   --********************************************************************--
   type NOTIFYSCROLL is record    -- notscr --
    hwndCnr   :  HWND        ;
    lScrollInc:  LONG        ;
    fScroll   :  ULONG       ;
   end record; -- NOTIFYSCROLL;
   type PNOTIFYSCROLL is access all NOTIFYSCROLL;

   --********************************************************************--
   -- Data structure for CN_REALLOCPSZ                                   --
   --********************************************************************--
subtype PPSZ is PVOID;
   type CNREDITDATA is record    -- cnredat --
    cb         :  ULONG        ;
    hwndCnr    :  HWND         ;
    pRecord    :  PRECORDCORE  ;
    p_FieldInfo:  PFIELDINFO   ;
    ppszText   :  PPSZ         ;  -- address of PSZ
    cbText     :  ULONG        ;  -- size of the new text
    id         :  ULONG        ;
   end record; -- CNREDITDATA;
   type PCNREDITDATA is access all CNREDITDATA;

   --********************************************************************--
   -- Data structure for CM_PAINTBACKGROUND                              --
   --********************************************************************--
   type OWNERBACKGROUND is record    -- ownbckg --
    h_wnd        :  HWND    ;
    h_ps         :  HPS     ;
    rclBackground:  RECTL   ;
    idWindow     :  LONG    ;
   end record; -- OWNERBACKGROUND;
   type POWNERBACKGROUND is access all OWNERBACKGROUND;

   --********************************************************************--
   -- Data structure used as part of WM_DRAWITEM                         --
   --********************************************************************--
   type CNRDRAWITEMINFO is record    -- cnrdrwii --
    pRecord    :  PRECORDCORE  ;
    p_FieldInfo:  PFIELDINFO   ;
   end record; -- CNRDRAWITEMINFO;
   type PCNRDRAWITEMINFO is access all CNRDRAWITEMINFO;

   --********************************************************************--
   -- Message parameter flags                                            --
   --********************************************************************--
CMA_TOP               :constant Long :=   16#0001#;  -- Place at top of zorder
CMA_BOTTOM            :constant Long :=   16#0002#;  -- Place at bottom of zorder
CMA_LEFT              :constant Long :=   16#0004#;
CMA_RIGHT             :constant Long :=   16#0008#;

CMA_FIRST             :constant Long :=   16#0010#;   -- Add record as first
CMA_LAST              :constant Long :=   16#0020#;
CMA_END               :constant Long :=   16#0040#;   -- Add record to end of list
CMA_PREV              :constant Long :=   16#0080#;
CMA_NEXT              :constant Long :=   16#0100#;

CMA_HORIZONTAL        :constant Long :=   16#0200#;
CMA_VERTICAL          :constant Long :=   16#0400#;
CMA_ICON              :constant Long :=   16#0800#;
CMA_TEXT              :constant Long :=   16#1000#;
CMA_PARTIAL           :constant Long :=   16#2000#;
CMA_COMPLETE          :constant Long :=   16#4000#;

CMA_PARENT            :constant Long :=   16#0001#;
CMA_FIRSTCHILD        :constant Long :=   16#0002#;
CMA_LASTCHILD         :constant Long :=   16#0004#;

CMA_CNRTITLE          :constant Long :=   16#0001#;  -- Container title
CMA_DELTA             :constant Long :=   16#0002#;  -- Application defined
CMA_FLWINDOWATTR      :constant Long :=   16#0004#;  -- Container attributes
CMA_LINESPACING       :constant Long :=   16#0008#;
CMA_PFIELDINFOLAST    :constant Long :=   16#0010#;   -- Ptr to last column in

CMA_PSORTRECORD       :constant Long :=   16#0020#;   -- Pointer to sort function
CMA_PTLORIGIN         :constant Long :=   16#0040#;   -- Lower left origin
CMA_SLBITMAPORICON    :constant Long :=   16#0080#;   -- Size  of bitmap
CMA_XVERTSPLITBAR     :constant Long :=   16#0100#;   -- Splitbar position
CMA_PFIELDINFOOBJECT  :constant Long :=   16#0200#;   -- Pointer to IN-USE
                                                      -- emphasis column.

CMA_TREEICON          :constant Long :=     16#0400#;   -- Icon for tree node
CMA_TREEBITMAP        :constant Long :=     16#0800#;   -- bitmap for tree node
CMA_CXTREEINDENT      :constant Long :=     16#1000#;   -- indent for children
CMA_CXTREELINE        :constant Long :=     16#2000#;   -- thickness of tree line
CMA_SLTREEBITMAPORICON:constant Long :=     16#4000#;   -- size of icon of tree node

CMA_ITEMORDER         :constant Long :=   16#0001#;     -- QueryRecord search flags
CMA_WINDOW            :constant Long :=   16#0002#;
CMA_WORKSPACE         :constant Long :=   16#0004#;
CMA_ZORDER            :constant Long :=   16#0008#;

CMA_DELTATOP          :constant Long :=   16#0001#;     -- Industrial - top delta
CMA_DELTABOT          :constant Long :=   16#0002#;     -- Industrial - bottom delt
CMA_DELTAHOME         :constant Long :=   16#0004#;     -- Industrial - top of list
CMA_DELTAEND          :constant Long :=   16#0008#;     -- Industrial - end of list

CMA_NOREPOSITION      :constant Long :=   16#0001#;     -- InvalidateRecord flags
CMA_REPOSITION        :constant Long :=   16#0002#;
CMA_TEXTCHANGED       :constant Long :=   16#0004#;
CMA_ERASE             :constant Long :=   16#0008#;

CMA_FREE              :constant Long :=   16#0001#;
CMA_INVALIDATE        :constant Long :=   16#0002#;

   --********************************************************************--
   --                                                                    --
   --                            S L I D E R                             --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- Define messages for the slider control                             --
   --********************************************************************--
 SLM_ADDDETENT            :constant Long := 16#0369#;  -- Add detent niche
 SLM_QUERYDETENTPOS       :constant Long := 16#036a#;  -- Query position of detent
 SLM_QUERYSCALETEXT       :constant Long := 16#036b#;  -- Query text at tick number
 SLM_QUERYSLIDERINFO      :constant Long := 16#036c#;  -- Query slider information
 SLM_QUERYTICKPOS         :constant Long := 16#036d#;  -- Query position of tick
 SLM_QUERYTICKSIZE        :constant Long := 16#036e#;  -- Query size of tick
 SLM_REMOVEDETENT         :constant Long := 16#036f#;  -- Remove detent niche
 SLM_SETSCALETEXT         :constant Long := 16#0370#;  -- Set text above tick
 SLM_SETSLIDERINFO        :constant Long := 16#0371#;  -- Set slider parameters
 SLM_SETTICKSIZE          :constant Long := 16#0372#;  -- Set size of tick
 SLN_CHANGE               :constant Long := 1 ;        -- Slider position changed
 SLN_SLIDERTRACK          :constant Long := 2 ;        -- Slider dragged by user
 SLN_SETFOCUS             :constant Long := 3 ;        -- Slider gaining focus
 SLN_KILLFOCUS            :constant Long := 4 ;        -- Slider losing focus

   --********************************************************************--
   -- Slider control data structure                                      --
   --********************************************************************--
   type SLDCDATA is record    -- sldcd --
    cbSize            :  ULONG   ;   -- Size of control block
    usScale1Increments:  USHORT  ;   -- # of divisions on scale
    usScale1Spacing   :  USHORT  ;   -- Space in pels between increments
    usScale2Increments:  USHORT  ;   -- # of divisions on scale
    usScale2Spacing   :  USHORT  ;   -- Space in pels between increments
   end record; -- SLDCDATA;
   type PSLDCDATA is access all SLDCDATA;

   --********************************************************************--
   -- Slider control style flag definition                               --
   --********************************************************************--
 SLS_HORIZONTAL         :constant Long := 16#00000000#; -- Orient slider horizontally
 SLS_VERTICAL           :constant Long := 16#00000001#; -- Orient slider vertically
 SLS_CENTER             :constant Long := 16#00000000#; -- Center shaft in window
 SLS_BOTTOM             :constant Long := 16#00000002#; -- Offset shaft to bottom (H)
 SLS_TOP                :constant Long := 16#00000004#; -- Offset shaft to top (H)
 SLS_LEFT               :constant Long := 16#00000002#; -- Offset shaft to left (V)
 SLS_RIGHT              :constant Long := 16#00000004#; -- Offset shaft to right (V)
 SLS_SNAPTOINCREMENT    :constant Long := 16#00000008#; -- Snap to nearest increment
 SLS_BUTTONSBOTTOM      :constant Long := 16#00000010#; -- Add buttons at shaft bot.
 SLS_BUTTONSTOP         :constant Long := 16#00000020#; -- Add buttons at shaft top
 SLS_BUTTONSLEFT        :constant Long := 16#00000010#; -- Add buttons left of shaft
 SLS_BUTTONSRIGHT       :constant Long := 16#00000020#; -- Add buttons right of shaft
 SLS_OWNERDRAW          :constant Long := 16#00000040#; -- Owner draw some fields
 SLS_READONLY           :constant Long := 16#00000080#; -- Provide a read only slider
 SLS_RIBBONSTRIP        :constant Long := 16#00000100#; -- Provide a ribbon strip
 SLS_HOMEBOTTOM         :constant Long := 16#00000000#; -- Set home position at bot.
 SLS_HOMETOP            :constant Long := 16#00000200#; -- Set home position at top
 SLS_HOMELEFT           :constant Long := 16#00000000#; -- Set home position at left
 SLS_HOMERIGHT          :constant Long := 16#00000200#; -- Set home position at right
 SLS_PRIMARYSCALE1      :constant Long := 16#00000000#; -- Scale 1 is primary scale
 SLS_PRIMARYSCALE2      :constant Long := 16#00000400#; -- Scale 2 is primary scale

   --********************************************************************--
   -- Message attributes for setting and querying slider components      --
   --********************************************************************--
 SMA_SCALE1                 :constant Long := 16#0001#;
 SMA_SCALE2                 :constant Long := 16#0002#;
 SMA_SHAFTDIMENSIONS        :constant Long := 16#0000#;
 SMA_SHAFTPOSITION          :constant Long := 16#0001#;
 SMA_SLIDERARMDIMENSIONS    :constant Long := 16#0002#;
 SMA_SLIDERARMPOSITION      :constant Long := 16#0003#;
 SMA_RANGEVALUE             :constant Long := 16#0000#;
 SMA_INCREMENTVALUE         :constant Long := 16#0001#;
 SMA_SETALLTICKS            :constant Long := 16#FFFF#;

   --********************************************************************--
   -- Ownerdraw flag definitions                                         --
   --********************************************************************--
  SDA_RIBBONSTRIP         :constant Long :=   16#0001#;
  SDA_SLIDERSHAFT         :constant Long :=   16#0002#;
  SDA_BACKGROUND          :constant Long :=   16#0003#;
  SDA_SLIDERARM           :constant Long :=   16#0004#;

   --********************************************************************--
   -- Error return codes                                                 --
   --********************************************************************--
 PMERR_UPDATE_IN_PROGRESS     :constant Long :=      16#1f06#;
 SLDERR_INVALID_PARAMETERS    :constant Long :=    -1 ;

   --********************************************************************--
   --                                                                    --
   --                         V A L U E   S E T                          --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- Define messages for the value set control                          --
   --********************************************************************--
 VM_QUERYITEM             :constant Long := 16#0375#;  -- Query item at location
 VM_QUERYITEMATTR         :constant Long := 16#0376#;  -- Query item attributes
 VM_QUERYMETRICS          :constant Long := 16#0377#;  -- Query metrics of control
 VM_QUERYSELECTEDITEM     :constant Long := 16#0378#;  -- Query selected item
 VM_SELECTITEM            :constant Long := 16#0379#;  -- Set selected item
 VM_SETITEM               :constant Long := 16#037a#;  -- Set item at location
 VM_SETITEMATTR           :constant Long := 16#037b#;  -- Set item attributes
 VM_SETMETRICS            :constant Long := 16#037c#;  -- Set metrics of control

 VN_SELECT                :constant Long := 120 ;     -- Item selected by user
 VN_ENTER                 :constant Long := 121 ;     -- Item entered by user
 VN_DRAGLEAVE             :constant Long := 122 ;     -- Drag left control
 VN_DRAGOVER              :constant Long := 123 ;     -- Drag is over item
 VN_DROP                  :constant Long := 124 ;     -- Drop occurred on item
 VN_DROPHELP              :constant Long := 125 ;     -- Request help for drop
 VN_INITDRAG              :constant Long := 126 ;     -- Drag initiated on item
 VN_SETFOCUS              :constant Long := 127 ;     -- Value set gaining focus
 VN_KILLFOCUS             :constant Long := 128 ;     -- Value set losing focus
 VN_HELP                  :constant Long := 129 ;     -- Help requested by user

   --********************************************************************--
   -- Value set control data structure                                   --
   --********************************************************************--
   type VSCDATA  is record   -- vscd --
    cbSize       :  ULONG   ;  -- Size of control block
    usRowCount   :  USHORT  ;  -- Number of rows in value set
    usColumnCount:  USHORT  ;  -- Number of columns in value set
   end record; -- VSCDATA;
   type PVSCDATA is access all VSCDATA;

   --********************************************************************--
   -- Value set drag initialization structure                            --
   --********************************************************************--
   type VSDRAGINIT is record    -- vsdinit --
    h_wnd   :  HWND    ;  -- Window handle of value set control
    x       :  LONG    ;  -- X coordinate of pointer on desktop
    y       :  LONG    ;  -- Y coordinate of pointer on desktop
    cx      :  LONG    ;  -- X offset from pointer hot spot
    cy      :  LONG    ;  -- Y offset from pointer hot spot
    usRow   :  USHORT  ;  -- Number of rows in value set
    usColumn:  USHORT  ;  -- Number of columns in value set
   end record; -- VSDRAGINIT;
   type PVSDRAGINIT is access all VSDRAGINIT;

      --*****************************************************************--
      -- Value set drag information structure                            --
      --*****************************************************************--
      type VSDRAGINFO is record    -- vsdinfo --
       p_DragInfo:  PDRAGINFO ;   -- Pointer to a drag info structure
       usRow     :  USHORT    ;   -- Number of rows in value set
       usColumn  :  USHORT    ;   -- Number of columns in value set
      end record; -- VSDRAGINFO;
      type PVSDRAGINFO is access all VSDRAGINFO;

   --********************************************************************--
   -- Value set query item text structure                                --
   --********************************************************************--
   type VSTEXT  is record   -- vstext --
    pszItemText:  PSZ       ;  -- Pointer to string for item text
    ulBufLen   :  ULONG     ;  -- Length of buffer to copy string into
   end record; -- VSTEXT;
   type PVSTEXT is access all VSTEXT;

   --********************************************************************--
   -- Value set control style flag definition                            --
   --********************************************************************--
    VS_BITMAP      :constant Long := 16#0001#; -- Default all items to bitmaps
    VS_ICON        :constant Long := 16#0002#; -- Default all items to icons
    VS_TEXT        :constant Long := 16#0004#; -- Default all items to text strings
    VS_RGB         :constant Long := 16#0008#; -- Default all items to color info
    VS_COLORINDEX  :constant Long := 16#0010#; -- Default all items to color indices
    VS_BORDER      :constant Long := 16#0020#; -- Add a border around the control
    VS_ITEMBORDER  :constant Long := 16#0040#; -- Add a border around each item
    VS_SCALEBITMAPS:constant Long := 16#0080#; -- Scale bitmaps to cell size
    VS_RIGHTTOLEFT :constant Long := 16#0100#; -- Support right to left ordering
    VS_OWNERDRAW   :constant Long := 16#0200#; -- Owner draws value set background

   --********************************************************************--
   -- Value set item attribute definition                                --
   --********************************************************************--
 VIA_BITMAP         :constant Long := 16#0001#;  -- If set; item contains a bitmap
 VIA_ICON           :constant Long := 16#0002#;  -- If set; item contains an icon
 VIA_TEXT           :constant Long := 16#0004#;  -- If set; item contains text string
 VIA_RGB            :constant Long := 16#0008#;  -- If set; item contains color value
 VIA_COLORINDEX     :constant Long := 16#0010#;  -- If set; item contains color index
 VIA_OWNERDRAW      :constant Long := 16#0020#;  -- If set; item is ownerdraw
 VIA_DISABLED       :constant Long := 16#0040#;  -- If set; item is unselectable
 VIA_DRAGGABLE      :constant Long := 16#0080#;  -- If set; item can be source of drag
 VIA_DROPONABLE     :constant Long := 16#0100#;  -- If set; item can be target of drop

   --********************************************************************--
   -- Message parameter attributes for sizing and spacing of items       --
   --********************************************************************--
 VMA_ITEMSIZE               :constant Long := 16#0001#;
 VMA_ITEMSPACING            :constant Long := 16#0002#;

   --********************************************************************--
   -- Ownerdraw flag definitions                                         --
   --********************************************************************--
 VDA_ITEM                  :constant Long :=  16#0001#;
 VDA_ITEMBACKGROUND        :constant Long :=  16#0002#;
 VDA_SURROUNDING           :constant Long :=  16#0003#;
 VDA_BACKGROUND            :constant Long :=  16#0004#;

   --********************************************************************--
   -- Error return codes                                                 --
   --********************************************************************--
 VSERR_INVALID_PARAMETERS    :constant Long :=    -1 ;

   --********************************************************************--
   --                                                                    --
   --                          N O T E B O O K                           --
   --                                                                    --
   --********************************************************************--

   --********************************************************************--
   -- Message ids                                                        --
   --********************************************************************--
 BKM_CALCPAGERECT           :constant Long := 16#0353#;-- Calc book/page rectangle
 BKM_DELETEPAGE             :constant Long := 16#0354#;-- Delete page(s)
 BKM_INSERTPAGE             :constant Long := 16#0355#;-- Insert page
 BKM_INVALIDATETABS         :constant Long := 16#0356#;-- Invalidate tab area
 BKM_TURNTOPAGE             :constant Long := 16#0357#;-- Turn to page
 BKM_QUERYPAGECOUNT         :constant Long := 16#0358#;-- Query number of pages
 BKM_QUERYPAGEID            :constant Long := 16#0359#;-- Query page identifier
 BKM_QUERYPAGEDATA          :constant Long := 16#035a#;-- Query page user data
 BKM_QUERYPAGEWINDOWHWND    :constant Long := 16#035b#;-- Query page window handle
 BKM_QUERYTABBITMAP         :constant Long := 16#035c#;-- Query tab bitmap handle
 BKM_QUERYTABTEXT           :constant Long := 16#035d#;-- Query tab text pointer
 BKM_SETDIMENSIONS          :constant Long := 16#035e#;-- Set tab/dogear dimensions
 BKM_SETPAGEDATA            :constant Long := 16#035f#;-- Set page user data
 BKM_SETPAGEWINDOWHWND      :constant Long := 16#0360#;-- Set page window handle
 BKM_SETSTATUSLINETEXT      :constant Long := 16#0361#;-- Set status line text
 BKM_SETTABBITMAP           :constant Long := 16#0362#;-- Set tab bitmap
 BKM_SETTABTEXT             :constant Long := 16#0363#;-- Set tab text
 BKM_SETNOTEBOOKCOLORS      :constant Long := 16#0364#;-- Set Notebook colors
 BKM_QUERYPAGESTYLE         :constant Long := 16#0365#;-- Query page style
 BKM_QUERYSTATUSLINETEXT    :constant Long := 16#0366#;-- Query status line text

 BKN_PAGESELECTED          :constant Long :=  130 ;   -- New page selected by user
 BKN_NEWPAGESIZE           :constant Long :=  131 ;   -- App page size changed
 BKN_HELP                  :constant Long :=  132 ;   -- Help notification
 BKN_PAGEDELETED           :constant Long :=  133 ;   -- Page deleted notification

   --********************************************************************--
   -- Page deletion flags (usDeleteFlag)                                 --
   --********************************************************************--
 BKA_ALL                   :constant Long :=  16#0001#;  -- all pages
 BKA_SINGLE                :constant Long :=  16#0002#;  -- single page
 BKA_TAB                   :constant Long :=  16#0004#;  -- minor/major section

   --********************************************************************--
   -- Page insertion/query order (usPageOrder; usQueryOrder)             --
   --********************************************************************--
  BKA_LAST              :constant Long :=     16#0002#;  -- Insert/Query last page
  BKA_FIRST             :constant Long :=     16#0004#;  -- Insert/Query first page
  BKA_NEXT              :constant Long :=     16#0008#;  -- Insert/Query after page
  BKA_PREV              :constant Long :=     16#0010#;  -- Insert/Query before page
  BKA_TOP               :constant Long :=     16#0020#;  -- Query topmost page

   --********************************************************************--
   -- Notebook region types (usBookRegion; usType)                       --
   --********************************************************************--
 BKA_MAJORTAB              :constant Long :=  16#0001#;  -- Major Tab
 BKA_MINORTAB              :constant Long :=  16#0002#;  -- Minor Tab
 BKA_PAGEBUTTON            :constant Long :=  16#0100#;  -- Page Turning Button

   --********************************************************************--
   -- Page insertion/query styles (usPageStyle;usQueryEnd)               --
   --********************************************************************--
 BKA_STATUSTEXTON          :constant Long :=  16#0001#;  -- status area text
 BKA_MAJOR                 :constant Long :=  16#0040#;  -- Major Tab
 BKA_MINOR                 :constant Long :=  16#0080#;  -- Minor Tab
 BKA_AUTOPAGESIZE          :constant Long :=  16#0100#;  -- Page window position/size
 BKA_END                   :constant Long :=  16#0200#;  -- Query to end of book

   --********************************************************************--
   -- Tab window contents (usTabDisplay)                                 --
   --********************************************************************--
 BKA_TEXT                   :constant Long := 16#0400#;  -- text data
 BKA_BITMAP                 :constant Long := 16#0800#;  -- bitmap

   --********************************************************************--
   -- Notebook window styles (ulNotebookStyles)                          --
   --********************************************************************--

   --********************************************************************--
   -- Backpage Orientation                                               --
   --********************************************************************--
BKS_BACKPAGESBR            :constant Long :=  16#00000001#;  -- Bottom right
BKS_BACKPAGESBL            :constant Long :=  16#00000002#;  -- Bottom left
BKS_BACKPAGESTR            :constant Long :=  16#00000004#;  -- Top right
BKS_BACKPAGESTL            :constant Long :=  16#00000008#;  -- Top left

   --********************************************************************--
   -- Major Tab Side                                                     --
   --********************************************************************--
 BKS_MAJORTABRIGHT         :constant Long :=  16#00000010#;  -- Major tabs right
 BKS_MAJORTABLEFT          :constant Long :=  16#00000020#;  -- Major tabs left
 BKS_MAJORTABTOP           :constant Long :=  16#00000040#;  -- Major tabs top
 BKS_MAJORTABBOTTOM        :constant Long :=  16#00000080#;  -- Major tabs bottom

   --********************************************************************--
   -- Tab Type                                                           --
   --********************************************************************--
 BKS_SQUARETABS            :constant Long :=  16#00000000#;  -- Square edged tabs
 BKS_ROUNDEDTABS           :constant Long :=  16#00000100#;  -- Round edged tabs
 BKS_POLYGONTABS           :constant Long :=  16#00000200#;  -- Polygon edged tabs

   --********************************************************************--
   -- Binding type                                                       --
   --********************************************************************--
  BKS_SOLIDBIND            :constant Long :=  16#00000000#;  -- Solid binding
  BKS_SPIRALBIND           :constant Long :=  16#00000400#;  -- Spiral binding

   --********************************************************************--
   -- Status line text justification                                     --
   --********************************************************************--
 BKS_STATUSTEXTLEFT       :constant Long :=   16#00000000#;  -- Left justify text
 BKS_STATUSTEXTRIGHT      :constant Long :=   16#00001000#;  -- Right justify text
 BKS_STATUSTEXTCENTER     :constant Long :=   16#00002000#;  -- Center text

   --********************************************************************--
   -- Tab text justification                                             --
   --********************************************************************--
 BKS_TABTEXTLEFT           :constant Long :=  16#00000000#;  -- Left justify tab text
 BKS_TABTEXTRIGHT          :constant Long :=  16#00004000#;  -- Right justify tab text
 BKS_TABTEXTCENTER         :constant Long :=  16#00008000#;  -- Center tab text

   --********************************************************************--
   -- Notebook color presentation param attributes                       --
   --********************************************************************--
 BKA_BACKGROUNDPAGECOLORINDEX    :constant Long := 16#0001#; -- Page Background
 BKA_BACKGROUNDPAGECOLOR         :constant Long := 16#0002#;
 BKA_BACKGROUNDMAJORCOLORINDEX   :constant Long := 16#0003#; -- Major Tab Background
 BKA_BACKGROUNDMAJORCOLOR        :constant Long := 16#0004#;
 BKA_BACKGROUNDMINORCOLORINDEX   :constant Long := 16#0005#; -- Minor Tab Background
 BKA_BACKGROUNDMINORCOLOR        :constant Long := 16#0006#;
 BKA_FOREGROUNDMAJORCOLORINDEX   :constant Long := 16#0007#; -- Major Tab Text
 BKA_FOREGROUNDMAJORCOLOR        :constant Long := 16#0008#;
 BKA_FOREGROUNDMINORCOLORINDEX   :constant Long := 16#0009#; -- Minor Tab Text
 BKA_FOREGROUNDMINORCOLOR        :constant Long := 16#000A#;

   --********************************************************************--
   -- Error message ids                                                  --
   --********************************************************************--
 BOOKERR_INVALID_PARAMETERS     :constant Long := -1; -- Invalid parameters

   --********************************************************************--
   -- BKM_QUERYTABTEXT and BKM_QUERYSTATUSLINETEXT message structure     --
   --********************************************************************--
   type BOOKTEXT is record    -- booktxt --
    pString:  PSZ    ;                        -- ptr to string buffer
    textLen:  ULONG  ;                        -- length of string to query
   end record; -- BOOKTEXT;
   type PBOOKTEXT is access all BOOKTEXT;

   --********************************************************************--
   -- BKN_PAGEDELETED notify message structure                           --
   --********************************************************************--
   type DELETENOTIFY  is record   -- delntfy --
    hwndBook     :  HWND    ;     -- Notebook window handle
    hwndPage     :  HWND    ;     -- App. page window handle
    ulAppPageData:  ULONG   ;     -- App. page data
    hbmTab       :  HBITMAP ;     -- App. tab bitmap handle
   end record; -- DELETENOTIFY;
   type PDELETENOTIFY is access all DELETENOTIFY;

   --********************************************************************--
   -- BKN_PAGESELECTED notify message structure                          --
   --********************************************************************--
   type PAGESELECTNOTIFY is record    -- pgsntfy --
    hwndBook   :  HWND    ;  -- Notebook window handle
    ulPageIdCur:  ULONG   ;  -- Previous top page id
    ulPageIdNew:  ULONG   ;  -- New top Page id
   end record; -- PAGESELECTNOTIFY;
   type PPAGESELECTNOTIFY is access all PAGESELECTNOTIFY;
end os2.dlg ;
