
-- ͻ
--        D E S I G N   E N G I N E R I N G              DS        
--             S O F T W A R E                           ͼ        
--                                                                    
--         Package     Os2.Pmhp                                       
--                                                                    
--         Author :  Leonid Dulman     1997                           
--                                                                    
--              GNU ADA RUNTIME LIBRARY (GNARL) COMPONENTS            
--                                                                    
--              Base PM window function                               
--                                                                    
-- ͼ

-- with Os2.pmsc ;            use Os2.pmsc  ;
with Interfaces.C;         use Interfaces.C;
with Interfaces.C.Strings; use Interfaces.C.Strings;

package Os2.PMhp is
pragma Preelaborate (PMhp);

--    HelpSubTable entry structure

subtype HELPSUBTABLE is USHORT;
type    PHELPSUBTABLE is access all HELPSUBTABLE;

--    HelpTable entry structure
--
--     Pack the structure HELPTABLE so that it is identical in the 32-Bit
--     and 16-Bit world.  We have to do this because the HelpTable can
--     reside either in memory or in the application's resources.

   type HELPTABLE is record   -- ht
      idAppWindow     :USHORT           ;
      phstHelpSubTable:PHELPSUBTABLE    ;
      idExtPanel      :USHORT           ;
   end record;
   type PHELPTABLE is access all HELPTABLE;


   --**********************************************************************
   -- IPF Initialization Structure used on the
   -- WinCreateHelpInstance() call.
   --**********************************************************************

   type HELPINIT is record     -- hinit 32 bit
      cb                      :ULONG        ;
      ulReturnCode            :ULONG        ;
      pszTutorialName         :PSZ          ;
      phtHelpTable            :PHELPTABLE   ;
      hmodHelpTableModule     :HMODULE      ;
      hmodAccelActionBarModule:HMODULE      ;
      idAccelTable            :ULONG        ;
      idActionBar             :ULONG        ;
      pszHelpWindowTitle      :PSZ          ;
      fShowPanelId            :ULONG        ;
      pszHelpLibraryName      :PSZ          ;
   end record ;
   type PHELPINIT is access all HELPINIT;


   --**********************************************************************
   -- Search parent chain indicator for HM_SET_ACTIVE_WINDOW message.
   --**********************************************************************
   HWND_PARENT : constant HWND := 0;

   --**********************************************************************
   -- Constants used to define whether user wants to display panel using
   -- panel number or panel name.
   --**********************************************************************
    HM_RESOURCEID          :constant Long :=   0 ;
    HM_PANELNAME           :constant Long :=   1 ;

    HMPANELTYPE_NUMBER     :constant Long :=   0 ;
    HMPANELTYPE_NAME       :constant Long :=   1 ;

   --**********************************************************************
   -- Constants used to define how the panel IDs are displayed on
   -- help panels.
   --**********************************************************************
    CMIC_HIDE_PANEL_ID     :constant Long :=   16#0000#;
    CMIC_SHOW_PANEL_ID     :constant Long :=   16#0001#;
    CMIC_TOGGLE_PANEL_ID   :constant Long :=   16#0002#;

   -- Window Help API declarations.

function  WinDestroyHelpInstance(hwndHelpInstance:HWND ) return   BOOL ;
pragma Import(c,WinDestroyHelpInstance, Link_name=>"_WinDestroyHelpInstance");

function  WinCreateHelpInstance(h_ab:HAB;
                                phinitHMInitStructure:PHELPINIT ) return    HWND ;
pragma Import(c,WinCreateHelpInstance, Link_name=>"_WinCreateHelpInstance");

function  WinAssociateHelpInstance(hwndHelpInstance:HWND ;
                                   hwndApp         :HWND  ) return    BOOL ;
pragma Import(c,WinAssociateHelpInstance, Link_name=>"_WinAssociateHelpInstance");

function  WinQueryHelpInstance( hwndApp:HWND) return    HWND ;
pragma Import(c,WinQueryHelpInstance, Link_name=>"_WinQueryHelpInstance");

function  WinLoadHelpTable(hwndHelpInstance:HWND    ;
                           idHelpTable     :ULONG   ;
                           Module          :HMODULE ) return    BOOL ;
pragma Import(c,WinLoadHelpTable, Link_name=>"_WinLoadHelpTable");

function  WinCreateHelpTable(hwndHelpInstance:HWND       ;
                             phtHelpTable    :PHELPTABLE ) return  BOOL ;
pragma Import(c,WinCreateHelpTable, Link_name=>"_WinCreateHelpTable");

   -- IPF message base.

 HM_MSG_BASE    :constant Long :=   16#0220#;

   -- Messages applications can send to the IPF.

HM_DISMISS_WINDOW                :constant Long :=  HM_MSG_BASE+16#0001#;
HM_DISPLAY_HELP                  :constant Long :=  HM_MSG_BASE+16#0002#;
HM_EXT_HELP                      :constant Long :=  HM_MSG_BASE+16#0003#;
HM_GENERAL_HELP                  :constant Long :=  HM_EXT_HELP;
HM_SET_ACTIVE_WINDOW             :constant Long :=  HM_MSG_BASE+16#0004#;
HM_LOAD_HELP_TABLE               :constant Long :=  HM_MSG_BASE+16#0005#;
HM_CREATE_HELP_TABLE             :constant Long :=  HM_MSG_BASE+16#0006#;
HM_SET_HELP_WINDOW_TITLE         :constant Long :=  HM_MSG_BASE+16#0007#;
HM_SET_SHOW_PANEL_ID             :constant Long :=  HM_MSG_BASE+16#0008#;
HM_REPLACE_HELP_FOR_HELP         :constant Long :=  HM_MSG_BASE+16#0009#;
HM_REPLACE_USING_HELP            :constant Long :=  HM_REPLACE_HELP_FOR_HELP;
HM_HELP_INDEX                    :constant Long :=  HM_MSG_BASE+16#000a#;
HM_HELP_CONTENTS                 :constant Long :=  HM_MSG_BASE+16#000b#;
HM_KEYS_HELP                     :constant Long :=  HM_MSG_BASE+16#000c#;
HM_SET_HELP_LIBRARY_NAME         :constant Long :=  HM_MSG_BASE+16#000d#;

HM_SET_OBJCOM_WINDOW             :constant Long :=  HM_MSG_BASE+16#0018#;
HM_UPDATE_OBJCOM_WINDOW_CHAIN    :constant Long :=  HM_MSG_BASE+16#0019#;
HM_QUERY_DDF_DATA                :constant Long :=  HM_MSG_BASE+16#001a#;
HM_INVALIDATE_DDF_DATA           :constant Long :=  HM_MSG_BASE+16#001b#;
HM_QUERY                         :constant Long :=  HM_MSG_BASE+16#001c#;
HM_SET_COVERPAGE_SIZE            :constant Long :=  HM_MSG_BASE+16#001d#;

   -- Constants used to query the info from IPF in HM_QUERY message

   -- Hi word in lParam 1
HMQW_COVERPAGE             :constant Long :=  16#0001#;
HMQW_INDEX                 :constant Long :=  16#0002#;
HMQW_TOC                   :constant Long :=  16#0003#;
HMQW_SEARCH                :constant Long :=  16#0004#;
HMQW_VIEWPAGES             :constant Long :=  16#0005#;
HMQW_LIBRARY               :constant Long :=  16#0006#;
HMQW_VIEWPORT              :constant Long :=  16#0007#;
HMQW_OBJCOM_WINDOW         :constant Long :=  16#0008#;
HMQW_INSTANCE              :constant Long :=  16#0009#;
HMQW_ACTIVEVIEWPORT        :constant Long :=  16#000a#;
CONTROL_SELECTED           :constant Long :=  16#000b#;

HMQW_GROUP_VIEWPORT        :constant Long :=  16#00f1#;
HMQW_RES_VIEWPORT          :constant Long :=  16#00f2#;
USERDATA                   :constant Long :=  16#00f3#;

   -- Lo word in lParam1 of HMQW_VIEWPORT
 HMQVP_NUMBER               :constant Long := 16#0001#;
 HMQVP_NAME                 :constant Long := 16#0002#;
 HMQVP_GROUP                :constant Long := 16#0003#;

   -- Predefined Control IDs

 CTRL_PREVIOUS_ID      :constant USHORT := 16#0001#;
 CTRL_SEARCH_ID        :constant USHORT := 16#0002#;
 CTRL_PRINT_ID         :constant USHORT := 16#0003#;
 CTRL_INDEX_ID         :constant USHORT := 16#0004#;
 CTRL_CONTENTS_ID      :constant USHORT := 16#0005#;
 CTRL_BACK_ID          :constant USHORT := 16#0006#;
 CTRL_FORWARD_ID       :constant USHORT := 16#0007#;
 CTRL_TUTORIAL_ID      :constant USHORT := 16#00FF#;

 CTRL_USER_ID_BASE     :constant USHORT := 257;


   -- Messages the IPF sends to the applications active window
   -- as defined by the IPF.

HM_ERROR                          :constant Long := HM_MSG_BASE+16#000e#;
HM_HELPSUBITEM_NOT_FOUND          :constant Long := HM_MSG_BASE+16#000f#;
HM_QUERY_KEYS_HELP                :constant Long := HM_MSG_BASE+16#0010#;
HM_TUTORIAL                       :constant Long := HM_MSG_BASE+16#0011#;
HM_EXT_HELP_UNDEFINED             :constant Long := HM_MSG_BASE+16#0012#;
HM_GENERAL_HELP_UNDEFINED         :constant Long := HM_EXT_HELP_UNDEFINED ;
HM_ACTIONBAR_COMMAND              :constant Long := HM_MSG_BASE+16#0013#;
HM_INFORM                         :constant Long := HM_MSG_BASE+16#0014#;
HM_NOTIFY                         :constant Long := HM_MSG_BASE+16#0022#;
HM_SET_USERDATA                   :constant Long := HM_MSG_BASE+16#0023#;
HM_CONTROL                        :constant Long := HM_MSG_BASE+16#0024#;

   -- notify information for HM_NOTIFY

OPEN_COVERPAGE           :constant Long := 16#0001#;
OPEN_PAGE                :constant Long := 16#0002#;
SWAP_PAGE                :constant Long := 16#0003#;
OPEN_TOC                 :constant Long := 16#0004#;
OPEN_INDEX               :constant Long := 16#0005#;
OPEN_HISTORY             :constant Long := 16#0006#;
OPEN_SEARCH_HIT_LIST     :constant Long := 16#0007#;
OPEN_LIBRARY             :constant Long := 16#0008#;
HELP_REQUESTED           :constant Long := 16#0009#;

   -- HMERR_NO_FRAME_WND_IN_CHAIN - There is no frame window in the
   -- window chain from which to find or set the associated help
   -- instance.

HMERR_NO_FRAME_WND_IN_CHAIN       :constant Long :=   16#00001001#;

   -- HMERR_INVALID_ASSOC_APP_WND - The application window handle
   -- specified on the WinAssociateHelpInstance() call is not a valid
   -- window handle.

HMERR_INVALID_ASSOC_APP_WND   :constant Long :=   16#00001002#;

   -- HMERR_INVALID_ASSOC_HELP_INST - The help instance handle specified
   -- on the WinAssociateHelpInstance() call is not a valid
   -- window handle.

HMERR_INVALID_ASSOC_HELP_INST   :constant Long :=   16#00001003#;

   -- HMERR_INVALID_DESTROY_HELP_INST - The window handle specified
   -- as the help instance to destroy is not of the help instance class.

HMERR_INVALID_DESTROY_HELP_INST  :constant Long :=  16#00001004#;

   -- HMERR_NO_HELP_INST_IN_CHAIN - The parent or owner chain of the
   -- application window specified does not have a help instance
   -- associated with it.

HMERR_NO_HELP_INST_IN_CHAIN    :constant Long :=    16#00001005#;

   -- HMERR_INVALID_HELP_INSTANCE_HDL - The handle specified to be a
   -- help instance does not have the class name of a IPF
   -- help instance.

HMERR_INVALID_HELP_INSTANCE_HDL  :constant Long :=     16#00001006#;

   -- HMERR_INVALID_QUERY_APP_WND - The application window specified on
   -- a WinQueryHelpInstance() call is not a valid window handle.

HMERR_INVALID_QUERY_APP_WND      :constant Long :=       16#00001007#;

   -- HMERR_HELP_INST_CALLED_INVALID -  The handle of the help instance
   -- specified on an API call to the IPF does not have the
   -- class name of an IPF help instance.

HMERR_HELP_INST_CALLED_INVALID      :constant Long :=   16#00001008#;
HMERR_HELPTABLE_UNDEFINE            :constant Long :=   16#00001009#;
HMERR_HELP_INSTANCE_UNDEFINE        :constant Long :=   16#0000100a#;
HMERR_HELPITEM_NOT_FOUND            :constant Long :=   16#0000100b#;
HMERR_INVALID_HELPSUBITEM_SIZE      :constant Long :=   16#0000100c#;
HMERR_HELPSUBITEM_NOT_FOUND         :constant Long :=   16#0000100d#;

   -- HMERR_INDEX_NOT_FOUND - No index in library file.

HMERR_INDEX_NOT_FOUND       :constant Long :=       16#00002001#;

   -- HMERR_CONTENT_NOT_FOUND - Library file does not have any contents.

HMERR_CONTENT_NOT_FOUND      :constant Long :=    16#00002002#;

   -- HMERR_OPEN_LIB_FILE     - Cannot open library file

HMERR_OPEN_LIB_FILE     :constant Long :=           16#00002003#;

   -- HMERR_READ_LIB_FILE     - Cannot read library file

HMERR_READ_LIB_FILE      :constant Long :=     16#00002004#;

   -- HMERR_CLOSE_LIB_FILE    - Cannot close library file

HMERR_CLOSE_LIB_FILE      :constant Long :=         16#00002005#;

   -- HMERR_INVALID_LIB_FILE  - Improper library file provided

HMERR_INVALID_LIB_FILE     :constant Long :=            16#00002006#;

   -- HMERR_NO_MEMORY - Unable to allocate the requested amount of memory.

HMERR_NO_MEMORY       :constant Long :=   16#00002007#;

   -- HMERR_ALLOCATE_SEGMENT - Unable
   -- to allocate a segment of memory for memory allocation requested
   -- from the IPF.

HMERR_ALLOCATE_SEGMENT       :constant Long :=   16#00002008#;

   -- HMERR_FREE_MEMORY - Unable to free allocated  memory

HMERR_FREE_MEMORY      :constant Long :=    16#00002009#;

   -- HMERR_PANEL_NOT_FOUND  - Unable
   -- to find a help panel requested to help manager

HMERR_PANEL_NOT_FOUND      :constant Long :=         16#00002010#;

   -- HMERR_DATABASE_NOT_OPEN - Unable to read the unopened database

HMERR_DATABASE_NOT_OPEN      :constant Long :=    16#00002011#;

   -- HMERR_DDL_ERROR - Unable to load resource dll

HMERR_LOAD_DLL       :constant Long :=             16#00002013#;

   -- AC Viewport stucture definitions

   type ACVP is record       -- acvp
      cb        : ULONG ;
      h_AB      : HAB   ;
      h_mq      : HMQ   ;
      ObjectID  : ULONG ;            -- object identifier
      hWndParent: HWND  ;          -- IPF viewport client handle
      hWndOwner : HWND  ;           -- IPF viewport client handle
      hWndACVP  : HWND  ;            -- applications frame window hwnd
  end record;
  type PACVP is access all ACVP;

   -- DDF defines and prototypes

  PMDDF_H :constant Long := 0;

      -- Define Handle to DDF

      subtype HDDF is PVOID;

      -- DdfHyperText Flags
REFERENCE_BY_ID         :constant Long :=   0;
REFERENCE_BY_RES        :constant Long :=   1;

      -- DdfBeginList formatting flags
HMBT_NONE              :constant Long :=  1;
HMBT_ALL               :constant Long :=  2;
HMBT_FIT               :constant Long :=  3;

HMLS_SINGLELINE        :constant Long :=  1;
HMLS_DOUBLELINE        :constant Long :=  2;

      -- DdfBitmap alignment flags
ART_RUNIN                :constant Long :=  16#10#;
ART_LEFT                 :constant Long :=  16#01#;
ART_RIGHT                :constant Long :=  16#02#;
ART_CENTER               :constant Long :=  16#04#;

      -- DdfSetColor Color Flag
CLR_UNCHANGED           :constant Long :=  -6;

      -- DDF API declarations.

function  DdfInitialize (hwndHelpInstance:HWND  ;
                         cbBuffer        :ULONG ;
                         ulIncrement     :ULONG ) return   HDDF ;
pragma Import(c,DdfInitialize, Link_name=>"_DdfInitialize");

function  DdfPara ( h_ddf:HDDF) return  BOOL ;
pragma Import(c,DdfPara, Link_name=>"_DdfPara");

function  DdfSetFormat (h_ddf      :HDDF ;
                        fFormatType:ULONG) return       BOOL ;
pragma Import(c,DdfSetFormat, Link_name=>"_DdfSetFormat");

function  DdfSetTextAlign (h_ddf :HDDF  ;
                           fAlign:ULONG ) return       BOOL ;
pragma Import(c,DdfSetTextAlign, Link_name=>"_DdfSetTextAlign");

function  DdfSetColor (h_ddf     :HDDF  ;
                       fBackColor:COLOR ;
                       fForColor :COLOR ) return       BOOL ;
pragma Import(c,DdfSetColor, Link_name=>"_DdfSetColor");

function  DdfInform (h_ddf          :HDDF  ;
                     pszText        :PSZ   ;
                     resInformNumber:ULONG ) return       BOOL ;
pragma Import(c,DdfInform, Link_name=>"_DdfInform");

function  DdfSetFontStyle (h_ddf     :HDDF  ;
                           fFontStyle:ULONG ) return       BOOL ;
pragma Import(c,DdfSetFontstyle, Link_name=>"_DdfSetFontstyle");

function  DdfHyperText (h_ddf         :HDDF  ;
                        pszText       :PSZ   ;
                        pszReference  :PSZ   ;
                        fReferenceType:ULONG ) return       BOOL ;
pragma Import(c,DdfHyperText, Link_name=>"_DdfHyperText");

function  DdfBeginList (h_ddf     :HDDF  ;
                        ulWidthDT :ULONG ;
                        fBreakType:ULONG ;
                        fSpacing  :ULONG ) return       BOOL ;
pragma Import(c,DdfBeginList, Link_name=>"_DdfBeginList");

function  DdfListItem (h_ddf         :HDDF ;
                       pszTerm       :PSZ  ;
                       pszDescription:PSZ  ) return       BOOL ;
pragma Import(c,DdfListItem, Link_name=>"_DdfListItem");

function  DdfEndList (h_ddf:HDDF ) return       BOOL ;
pragma Import(c,DdfEndList, Link_name=>"_DdfEndList");

function  DdfMetafile (h_ddf   :HDDF   ;
                       h_mf    :HMF    ;
                       prclRect:PRECTL ) return       BOOL ;
pragma Import(c,DdfMetafile, Link_name=>"_DdfMetafile");

function  DdfText (h_ddf  :HDDF ;
                   pszText:PSZ  ) return       BOOL ;
pragma Import(c,DdfText, Link_name=>"_DdfText");

function  DdfSetFont (h_ddf      :HDDF  ;
                      pszFaceName:PSZ   ;
                      ulWidth    :ULONG ;
                      ulHeight   :ULONG ) return       BOOL ;
pragma Import(c,DdfSetFont, Link_name=>"_DdfSetFont");

function  DdfBitmap (h_ddf :HDDF    ;
                     hbm   :HBITMAP ;
                     fAlign:ULONG   ) return       BOOL ;
pragma Import(c,DdfBitmap, Link_name=>"_DdfBitmap");

      -- error codes returned by DDF API functions

HMERR_DDF_MEMORY                       :constant Long :=  16#3001#;
HMERR_DDF_ALIGN_TYPE                   :constant Long :=  16#3002#;
HMERR_DDF_BACKCOLOR                    :constant Long :=  16#3003#;
HMERR_DDF_FORECOLOR                    :constant Long :=  16#3004#;
HMERR_DDF_FONTSTYLE                    :constant Long :=  16#3005#;
HMERR_DDF_REFTYPE                      :constant Long :=  16#3006#;
HMERR_DDF_LIST_UNCLOSED                :constant Long :=  16#3007#;
HMERR_DDF_LIST_UNINITIALIZED           :constant Long :=  16#3008#;
HMERR_DDF_LIST_BREAKTYPE               :constant Long :=  16#3009#;
HMERR_DDF_LIST_SPACING                 :constant Long :=  16#300A#;
HMERR_DDF_HINSTANCE                    :constant Long :=  16#300B#;
HMERR_DDF_EXCEED_MAX_LENGTH            :constant Long :=  16#300C#;
HMERR_DDF_EXCEED_MAX_INC               :constant Long :=  16#300D#;
HMERR_DDF_INVALID_DDF                  :constant Long :=  16#300E#;
HMERR_DDF_FORMAT_TYPE                  :constant Long :=  16#300F#;
HMERR_DDF_INVALID_PARM                 :constant Long :=  16#3010#;
HMERR_DDF_INVALID_FONT                 :constant Long :=  16#3011#;
HMERR_DDF_SEVERE                       :constant Long :=  16#3012#;

end Os2.PMhp;
